@@ -15,9 +15,11 @@
  */
 package org.gradle.internal.service;
 
+import com.google.common.collect.ImmutableList;
 import org.gradle.internal.Factory;
 import org.gradle.internal.scan.UsedByScanPlugin;
 
+import java.lang.annotation.Annotation;
 import java.lang.reflect.Type;
 import java.util.List;
 
@@ -88,4 +90,45 @@ public interface ServiceRegistry extends ServiceLookup {
      * @throws ServiceLookupException On failure to lookup the specified service factory.
      */
     <T> T newInstance(Class<T> type) throws UnknownServiceException, ServiceLookupException;
+
+    ServiceRegistry EMPTY = new ServiceRegistry() {
+        @Override
+        public <T> T get(Class<T> serviceType) throws UnknownServiceException, ServiceLookupException {
+            throw emptyServiceRegistryException(serviceType);
+        }
+
+        @Override
+        public <T> List<T> getAll(Class<T> serviceType) throws ServiceLookupException {
+            return ImmutableList.of();
+        }
+
+        @Override
+        public Object get(Type serviceType) throws UnknownServiceException, ServiceLookupException {
+            throw emptyServiceRegistryException(serviceType);
+        }
+
+        @Override
+        public Object find(Type serviceType) throws ServiceLookupException {
+            return null;
+        }
+
+        @Override
+        public <T> Factory<T> getFactory(Class<T> type) throws UnknownServiceException, ServiceLookupException {
+            throw emptyServiceRegistryException(type);
+        }
+
+        private UnknownServiceException emptyServiceRegistryException(Type type) {
+            return new UnknownServiceException(type, "Nothing is available in the empty service registry.");
+        }
+
+        @Override
+        public <T> T newInstance(Class<T> type) throws UnknownServiceException, ServiceLookupException {
+            throw emptyServiceRegistryException(type);
+        }
+
+        @Override
+        public Object get(Type serviceType, Class<? extends Annotation> annotatedWith) throws UnknownServiceException, ServiceLookupException {
+            throw emptyServiceRegistryException(serviceType);
+        }
+    };
 }
