@@ -16,12 +16,11 @@
 
 package org.gradle.internal.work;
 
+import org.gradle.internal.Factory;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.resources.ProjectLeaseRegistry;
 import org.gradle.internal.resources.ResourceLock;
 
-import java.util.concurrent.Callable;
-
 public interface WorkerLeaseService extends WorkerLeaseRegistry, ProjectLeaseRegistry, Stoppable {
     /**
      * Returns the maximum number of worker leases that this service will grant at any given time. Note that the actual limit may vary over time but will never _exceed_ the value returned by this method.
@@ -29,28 +28,28 @@ public interface WorkerLeaseService extends WorkerLeaseRegistry, ProjectLeaseReg
     int getMaxWorkerCount();
 
     /**
-     * Runs a given {@link Callable} while the specified locks are being held, releasing
+     * Runs a given {@link Factory} while the specified locks are being held, releasing
      * the locks upon completion.  Blocks until the specified locks can be obtained.
      */
-    <T> T withLocks(Iterable<? extends ResourceLock> locks, Callable<T> action);
+    <T> T withLocks(Iterable<? extends ResourceLock> locks, Factory<T> factory);
 
     /**
      * Runs a given {@link Runnable} while the specified locks are being held, releasing
      * the locks upon completion.  Blocks until the specified locks can be obtained.
      */
-    void withLocks(Iterable<? extends ResourceLock> locks, Runnable action);
+    void withLocks(Iterable<? extends ResourceLock> locks, Runnable runnable);
 
     /**
-     * Runs a given {@link Callable} while the specified locks are released and then reacquire the locks
+     * Runs a given {@link Factory} while the specified locks are released and then reacquire the locks
      * upon completion.  If the locks cannot be immediately reacquired, the current worker lease will be released
      * and the method will block until the locks are reacquired.
      */
-    <T> T withoutLocks(Iterable<? extends ResourceLock> locks, Callable<T> action);
+    <T> T withoutLocks(Iterable<? extends ResourceLock> locks, Factory<T> factory);
 
     /**
      * Runs a given {@link Runnable} while the specified locks are released and then reacquire the locks
      * upon completion.  If the locks cannot be immediately reacquired, the current worker lease will be released
      * and the method will block until the locks are reacquired.
      */
-    void withoutLocks(Iterable<? extends ResourceLock> locks, Runnable action);
+    void withoutLocks(Iterable<? extends ResourceLock> locks, Runnable runnable);
 }
