@@ -29,22 +29,19 @@ import org.gradle.api.Describable;
 import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.Project;
 import org.gradle.api.Task;
-import org.gradle.api.internal.file.FileResolver;
+import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.TemporaryFileProvider;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.internal.project.taskfactory.TaskIdentity;
-import org.gradle.api.internal.tasks.ContextAwareTaskAction;
-import org.gradle.api.internal.tasks.DefaultPropertySpecFactory;
 import org.gradle.api.internal.tasks.DefaultTaskDependency;
 import org.gradle.api.internal.tasks.DefaultTaskDestroyables;
 import org.gradle.api.internal.tasks.DefaultTaskInputs;
 import org.gradle.api.internal.tasks.DefaultTaskLocalState;
 import org.gradle.api.internal.tasks.DefaultTaskOutputs;
 import org.gradle.api.internal.tasks.ImplementationAwareTaskAction;
-import org.gradle.api.internal.tasks.PropertySpecFactory;
+import org.gradle.api.internal.tasks.InputChangesAwareTaskAction;
 import org.gradle.api.internal.tasks.TaskContainerInternal;
 import org.gradle.api.internal.tasks.TaskDependencyInternal;
-import org.gradle.api.internal.tasks.TaskExecutionContext;
 import org.gradle.api.internal.tasks.TaskLocalStateInternal;
 import org.gradle.api.internal.tasks.TaskMutator;
 import org.gradle.api.internal.tasks.TaskStateInternal;
@@ -63,8 +60,10 @@ import org.gradle.api.tasks.TaskInstantiationException;
 import org.gradle.api.tasks.TaskLocalState;
 import org.gradle.internal.Factory;
 import org.gradle.internal.classloader.ClassLoaderHierarchyHasher;
+import org.gradle.internal.execution.history.changes.InputChangesInternal;
 import org.gradle.internal.extensibility.ExtensibleDynamicObject;
 import org.gradle.internal.logging.compatbridge.LoggingManagerInternalCompatibilityBridge;
+import org.gradle.internal.logging.slf4j.DefaultContextAwareTaskLogger;
 import org.gradle.internal.metaobject.DynamicObject;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.scripts.ScriptOrigin;
@@ -97,7 +96,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     private final ProjectInternal project;
 
-    private List<ContextAwareTaskAction> actions;
+    private List<InputChangesAwareTaskAction> actions;
 
     private boolean enabled = true;
 
@@ -123,7 +122,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     private final TaskStateInternal state;
 
-    private Logger logger = BUILD_LOGGER;
+    private Logger logger = new DefaultContextAwareTaskLogger(BUILD_LOGGER);
 
     private final TaskMutator taskMutator;
     private ObservableList observableActionList;
@@ -137,7 +136,6 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
     private LoggingManagerInternal loggingManager;
 
     private String toStringValue;
-    private final PropertySpecFactory specFactory;
 
     protected AbstractTask() {
         this(taskInfo());
@@ -163,12 +161,11 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         this.shouldRunAfter = new DefaultTaskDependency(tasks);
         this.services = project.getServices();
 
-        FileResolver fileResolver = project.getFileResolver();
         PropertyWalker propertyWalker = services.get(PropertyWalker.class);
+        FileCollectionFactory fileCollectionFactory = services.get(FileCollectionFactory.class);
         taskMutator = new TaskMutator(this);
-        specFactory = new DefaultPropertySpecFactory(this, fileResolver);
-        taskInputs = new DefaultTaskInputs(this, taskMutator, propertyWalker, specFactory);
-        taskOutputs = new DefaultTaskOutputs(this, taskMutator, propertyWalker, specFactory);
+        taskInputs = new DefaultTaskInputs(this, taskMutator, propertyWalker, fileCollectionFactory);
+        taskOutputs = new DefaultTaskOutputs(this, taskMutator, propertyWalker, fileCollectionFactory);
         taskDestroyables = new DefaultTaskDestroyables(taskMutator);
         taskLocalState = new DefaultTaskLocalState(taskMutator);
 
@@ -231,9 +228,9 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
     }
 
     @Override
-    public List<ContextAwareTaskAction> getTaskActions() {
+    public List<InputChangesAwareTaskAction> getTaskActions() {
         if (actions == null) {
-            actions = new ArrayList<ContextAwareTaskAction>(3);
+            actions = new ArrayList<InputChangesAwareTaskAction>(3);
         }
         return actions;
     }
@@ -593,17 +590,17 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         };
     }
 
-    private ContextAwareTaskAction convertClosureToAction(Closure actionClosure, String actionName) {
+    private InputChangesAwareTaskAction convertClosureToAction(Closure actionClosure, String actionName) {
         return new ClosureTaskAction(actionClosure, actionName);
     }
 
-    private ContextAwareTaskAction wrap(final Action<? super Task> action) {
+    private InputChangesAwareTaskAction wrap(final Action<? super Task> action) {
         return wrap(action, "unnamed action");
     }
 
-    private ContextAwareTaskAction wrap(final Action<? super Task> action, String actionName) {
-        if (action instanceof ContextAwareTaskAction) {
-            return (ContextAwareTaskAction) action;
+    private InputChangesAwareTaskAction wrap(final Action<? super Task> action, String actionName) {
+        if (action instanceof InputChangesAwareTaskAction) {
+            return (InputChangesAwareTaskAction) action;
         }
         return new TaskActionWrapper(action, actionName);
     }
@@ -618,7 +615,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
     }
 
-    private static class ClosureTaskAction implements ContextAwareTaskAction {
+    private static class ClosureTaskAction implements InputChangesAwareTaskAction {
         private final Closure closure;
         private final String actionName;
 
@@ -628,11 +625,11 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
 
         @Override
-        public void contextualise(TaskExecutionContext context) {
+        public void setInputChanges(InputChangesInternal inputChanges) {
         }
 
         @Override
-        public void releaseContext() {
+        public void clearInputChanges() {
         }
 
         @Override
@@ -669,7 +666,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
     }
 
-    private static class TaskActionWrapper implements ContextAwareTaskAction {
+    private static class TaskActionWrapper implements InputChangesAwareTaskAction {
         private final Action<? super Task> action;
         private final String maybeActionName;
 
@@ -684,16 +681,16 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
 
         @Override
-        public void contextualise(TaskExecutionContext context) {
-            if (action instanceof ContextAwareTaskAction) {
-                ((ContextAwareTaskAction) action).contextualise(context);
+        public void setInputChanges(InputChangesInternal inputChanges) {
+            if (action instanceof InputChangesAwareTaskAction) {
+                ((InputChangesAwareTaskAction) action).setInputChanges(inputChanges);
             }
         }
 
         @Override
-        public void releaseContext() {
-            if (action instanceof ContextAwareTaskAction) {
-                ((ContextAwareTaskAction) action).releaseContext();
+        public void clearInputChanges() {
+            if (action instanceof InputChangesAwareTaskAction) {
+                ((InputChangesAwareTaskAction) action).clearInputChanges();
             }
         }
 
@@ -876,9 +873,9 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
             return super.remove(wrap((Action<? super Task>) action));
         }
 
-        private Collection<ContextAwareTaskAction> transformToContextAwareTaskActions(Collection<Object> c) {
-            return Collections2.transform(c, new Function<Object, ContextAwareTaskAction>() {
-                public ContextAwareTaskAction apply(@Nullable Object input) {
+        private Collection<InputChangesAwareTaskAction> transformToContextAwareTaskActions(Collection<Object> c) {
+            return Collections2.transform(c, new Function<Object, InputChangesAwareTaskAction>() {
+                public InputChangesAwareTaskAction apply(@Nullable Object input) {
                     return wrap((Action<? super Task>) input);
                 }
             });
