@@ -28,13 +28,13 @@ import org.gradle.api.NamedDomainObjectProvider;
 import org.gradle.api.Namer;
 import org.gradle.api.Rule;
 import org.gradle.api.UnknownDomainObjectException;
-import org.gradle.api.UnknownTaskException;
 import org.gradle.api.internal.collections.CollectionEventRegister;
 import org.gradle.api.internal.collections.CollectionFilter;
 import org.gradle.api.internal.collections.ElementSource;
 import org.gradle.api.internal.plugins.DslObject;
-import org.gradle.api.internal.provider.AbstractProvider;
+import org.gradle.api.internal.provider.AbstractReadOnlyProvider;
 import org.gradle.api.internal.provider.ProviderInternal;
+import org.gradle.api.internal.provider.Providers;
 import org.gradle.api.provider.Provider;
 import org.gradle.api.reflect.TypeOf;
 import org.gradle.api.specs.Spec;
@@ -71,7 +71,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
     private final Instantiator instantiator;
     private final Namer<? super T> namer;
-    private final Index<T> index;
+    protected final Index<T> index;
 
     private final ContainerElementsDynamicObject elementsDynamicObject = new ContainerElementsDynamicObject();
 
@@ -79,8 +79,8 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     private final Set<String> applyingRulesFor = new HashSet<String>();
     private ImmutableActionSet<ElementInfo<T>> whenKnown = ImmutableActionSet.empty();
 
-    public DefaultNamedDomainObjectCollection(Class<? extends T> type, ElementSource<T> store, Instantiator instantiator, Namer<? super T> namer) {
-        super(type, store);
+    public DefaultNamedDomainObjectCollection(Class<? extends T> type, ElementSource<T> store, Instantiator instantiator, Namer<? super T> namer, CollectionCallbackActionDecorator callbackActionDecorator) {
+        super(type, store, callbackActionDecorator);
         this.instantiator = instantiator;
         this.namer = namer;
         this.index = new UnfilteredIndex<T>();
@@ -106,21 +106,16 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     @Override
-    public boolean add(final T o) {
-        return add(o, getEventRegister().getAddActions());
-    }
-
-    @Override
-    protected <I extends T> boolean add(final I o, Action<? super I> notification) {
-        final String name = namer.determineName(o);
+    protected <I extends T> boolean doAdd(I toAdd, Action<? super I> notification) {
+        final String name = namer.determineName(toAdd);
         if (index.get(name) == null) {
-            boolean added = super.add(o, notification);
+            boolean added = super.doAdd(toAdd, notification);
             if (added) {
-                whenKnown.execute(new ObjectBackedElementInfo<T>(name, o));
+                whenKnown.execute(new ObjectBackedElementInfo<T>(name, toAdd));
             }
             return added;
         } else {
-            handleAttemptToAddItemWithNonUniqueName(o);
+            handleAttemptToAddItemWithNonUniqueName(toAdd);
             return false;
         }
     }
@@ -133,6 +128,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
     @Override
     public boolean addAll(Collection<? extends T> c) {
+        assertMutable("addAll(Collection<T>)");
         boolean changed = super.addAll(c);
         if (changed) {
             for (T t : c) {
@@ -145,10 +141,11 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
     @Override
     public void addLater(final Provider<? extends T> provider) {
+        assertMutable("addLater(Provider)");
         super.addLater(provider);
         if (provider instanceof Named) {
             final Named named = (Named) provider;
-            index.putPending(named.getName(), (ProviderInternal<? extends T>) provider);
+            index.putPending(named.getName(), Providers.internal(provider));
             deferredElementKnown(named.getName(), provider);
         }
     }
@@ -167,7 +164,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     protected final void deferredElementKnown(String name, Provider<? extends T> provider) {
-        whenKnown.execute(new ProviderBackedElementInfo<T>(name, provider));
+        whenKnown.execute(new ProviderBackedElementInfo<T>(name, Providers.internal(provider)));
     }
 
     @Override
@@ -186,6 +183,16 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         index.remove(namer.determineName(t));
     }
 
+    @Override
+    protected void didRemove(ProviderInternal<? extends T> t) {
+        if (t instanceof Named) {
+            index.removePending(((Named) t).getName());
+        }
+        if (t instanceof AbstractDomainObjectCreatingProvider) {
+            ((AbstractDomainObjectCreatingProvider) t).removedBeforeRealized = true;
+        }
+    }
+
     /**
      * <p>Subclass hook for implementations wanting to throw an exception when an attempt is made to add an item with the same name as an existing item.</p>
      *
@@ -302,15 +309,11 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
     protected T removeByName(String name) {
         T it = getByName(name);
-        if (it != null) {
-            if (remove(it)) {
-                return it;
-            } else {
-                // unclear what the best thing to do here would be
-                throw new IllegalStateException(String.format("found '%s' with name '%s' but remove() returned false", it, name));
-            }
+        if (remove(it)) {
+            return it;
         } else {
-            return null;
+            // unclear what the best thing to do here would be
+            throw new IllegalStateException(String.format("found '%s' with name '%s' but remove() returned false", it, name));
         }
     }
 
@@ -323,13 +326,12 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     public T getByName(String name, Closure configureClosure) throws UnknownDomainObjectException {
-        T t = getByName(name);
-        ConfigureUtil.configure(configureClosure, t);
-        return t;
+        return getByName(name, ConfigureUtil.configureUsing(configureClosure));
     }
 
     @Override
     public T getByName(String name, Action<? super T> configureAction) throws UnknownDomainObjectException {
+        assertMutable("getByName(String, Action)");
         T t = getByName(name);
         configureAction.execute(t);
         return t;
@@ -340,7 +342,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     @Override
-    public NamedDomainObjectProvider<T> named(String name) throws UnknownTaskException {
+    public NamedDomainObjectProvider<T> named(String name) throws UnknownDomainObjectException {
         NamedDomainObjectProvider<? extends T> provider = findDomainObject(name);
         if (provider == null) {
             throw createNotFoundException(name);
@@ -348,6 +350,36 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         return Cast.uncheckedCast(provider);
     }
 
+    @Override
+    public NamedDomainObjectProvider<T> named(String name, Action<? super T> configurationAction) throws UnknownDomainObjectException {
+        assertMutable("named(String, Action)");
+        NamedDomainObjectProvider<T> provider = named(name);
+        provider.configure(configurationAction);
+        return provider;
+    }
+
+    @Override
+    public <S extends T> NamedDomainObjectProvider<S> named(String name, Class<S> type) throws UnknownDomainObjectException {
+        AbstractNamedDomainObjectProvider<S> provider = Cast.uncheckedCast(named(name));
+        Class<S> actual = provider.type;
+        if (!type.isAssignableFrom(actual)) {
+            throw createWrongTypeException(name, type, actual);
+        }
+        return provider;
+    }
+
+    protected InvalidUserDataException createWrongTypeException(String name, Class expected, Class actual) {
+        return new InvalidUserDataException(String.format("The domain object '%s' (%s) is not a subclass of the given type (%s).", name, actual.getCanonicalName(), expected.getCanonicalName()));
+    }
+
+    @Override
+    public <S extends T> NamedDomainObjectProvider<S> named(String name, Class<S> type, Action<? super S> configurationAction) throws UnknownDomainObjectException {
+        assertMutable("named(String, Class, Action)");
+        NamedDomainObjectProvider<S> provider = named(name, type);
+        provider.configure(configurationAction);
+        return provider;
+    }
+
     @Override
     public MethodAccess getAdditionalMethods() {
         return getElementsAsDynamicObject();
@@ -367,46 +399,23 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         return new NamedDomainObjectCollectionSchema() {
             @Override
             public Iterable<? extends NamedDomainObjectSchema> getElements() {
-                return Iterables.concat(
-                    Iterables.transform(index.asMap().entrySet(), new Function<Map.Entry<String, T>, NamedDomainObjectSchema>() {
-                        @Override
-                        public NamedDomainObjectSchema apply(final Map.Entry<String, T> e) {
-                            return new NamedDomainObjectSchema() {
-                                @Override
-                                public String getName() {
-                                    return e.getKey();
-                                }
+                // Simple scheme is to just present the public type of the container
+                return Iterables.transform(getNames(), new Function<String, NamedDomainObjectSchema>() {
+                    @Override
+                    public NamedDomainObjectSchema apply(final String name) {
+                        return new NamedDomainObjectSchema() {
+                            @Override
+                            public String getName() {
+                                return name;
+                            }
 
-                                @Override
-                                public TypeOf<?> getPublicType() {
-                                    // TODO: This returns the wrong public type for domain objects
-                                    // created with the eager APIs or added directly to the container.
-                                    // This can leak internal types.
-                                    // We do not currently keep track of the type used when creating
-                                    // a domain object (via create) or the type of the container when
-                                    // a domain object is added directly (via add).
-                                    return TypeOf.typeOf(new DslObject(e.getValue()).getDeclaredType());
-                                }
-                            };
-                        }
-                    }),
-                    Iterables.transform(index.getPendingAsMap().entrySet(), new Function<Map.Entry<String, ProviderInternal<? extends T>>, NamedDomainObjectSchema>() {
-                        @Override
-                        public NamedDomainObjectSchema apply(final Map.Entry<String, ProviderInternal<? extends T>> e) {
-                            return new NamedDomainObjectSchema() {
-                                @Override
-                                public String getName() {
-                                    return e.getKey();
-                                }
-
-                                @Override
-                                public TypeOf<?> getPublicType() {
-                                    return TypeOf.typeOf(e.getValue().getType());
-                                }
-                            };
-                        }
-                    })
-                );
+                            @Override
+                            public TypeOf<?> getPublicType() {
+                                return TypeOf.typeOf(getType());
+                            }
+                        };
+                    }
+                });
             }
         };
     }
@@ -478,7 +487,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
     protected UnknownDomainObjectException createNotFoundException(String name) {
         return new UnknownDomainObjectException(String.format("%s with name '%s' not found.", getTypeDisplayName(),
-                name));
+            name));
     }
 
     protected String getTypeDisplayName() {
@@ -578,6 +587,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         @Override
         public void clear() {
             map.clear();
+            pendingMap.clear();
         }
 
         @Override
@@ -731,9 +741,9 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
     private static class ProviderBackedElementInfo<T> implements ElementInfo<T> {
         private final String name;
-        private final Provider<? extends T> provider;
+        private final ProviderInternal<? extends T> provider;
 
-        ProviderBackedElementInfo(String name, Provider<? extends T> provider) {
+        ProviderBackedElementInfo(String name, ProviderInternal<? extends T> provider) {
             this.name = name;
             this.provider = provider;
         }
@@ -745,7 +755,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
         @Override
         public Class<?> getType() {
-            return ((ProviderInternal<?>) provider).getType();
+            return provider.getType();
         }
     }
 
@@ -781,20 +791,22 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     protected NamedDomainObjectProvider<? extends T> createExistingProvider(String name, T object) {
-        return Cast.uncheckedCast(getInstantiator().newInstance(ExistingNamedDomainObjectProvider.class, this, object, name));
+        return Cast.uncheckedCast(getInstantiator().newInstance(ExistingNamedDomainObjectProvider.class, this, name, new DslObject(object).getDeclaredType()));
     }
 
-    protected abstract class AbstractNamedDomainObjectProvider<I extends T> extends AbstractProvider<I> implements Named, NamedDomainObjectProvider<I> {
+    protected abstract class AbstractNamedDomainObjectProvider<I extends T> extends AbstractReadOnlyProvider<I> implements Named, NamedDomainObjectProvider<I> {
         private final String name;
+        private final Class<I> type;
 
-        protected AbstractNamedDomainObjectProvider(String name) {
+        protected AbstractNamedDomainObjectProvider(String name, Class<I> type) {
             this.name = name;
+            this.type = type;
         }
 
         @Nullable
         @Override
         public Class<I> getType() {
-            return (Class<I>) DefaultNamedDomainObjectCollection.this.getType();
+            return type;
         }
 
         @Override
@@ -804,7 +816,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
 
         @Override
         public boolean isPresent() {
-            return findDomainObject(name) != null;
+            return findDomainObject(getName()) != null;
         }
 
         @Override
@@ -814,24 +826,141 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     protected class ExistingNamedDomainObjectProvider<I extends T> extends AbstractNamedDomainObjectProvider<I> {
-        private final I object;
 
-        public ExistingNamedDomainObjectProvider(I object, String name) {
-            super(name);
-            this.object = object;
+        public ExistingNamedDomainObjectProvider(String name, Class type) {
+            super(name, type);
         }
 
         public void configure(Action<? super I> action) {
-            action.execute(object);
+            assertMutable("NamedDomainObjectProvider.configure(Action)");
+            withMutationDisabled(action).execute(get());
         }
 
         @Override
         public boolean isPresent() {
-            return true;
+            return getOrNull() != null;
         }
 
+        @Override
+        public I get() {
+            if (!isPresent()) {
+                throw domainObjectRemovedException(getName(), getType());
+            }
+            return super.get();
+        }
+
+        @Override
         public I getOrNull() {
-            return object;
+            return Cast.uncheckedCast(findByNameWithoutRules(getName()));
         }
     }
+
+    public abstract class AbstractDomainObjectCreatingProvider<I extends T> extends AbstractNamedDomainObjectProvider<I> {
+        private I object;
+        private RuntimeException failure;
+        protected ImmutableActionSet<I> onCreate;
+        private boolean removedBeforeRealized = false;
+
+        public AbstractDomainObjectCreatingProvider(String name, Class<I> type, @Nullable Action<? super I> configureAction) {
+            super(name, type);
+            this.onCreate = ImmutableActionSet.<I>empty().mergeFrom(getEventRegister().getAddActions());
+
+            if (configureAction != null) {
+                configure(configureAction);
+            }
+        }
+
+        @Override
+        public boolean isPresent() {
+            return findDomainObject(getName()) != null;
+        }
+
+        @Override
+        public void configure(final Action<? super I> action) {
+            assertMutable("NamedDomainObjectProvider.configure(Action)");
+            Action<? super I> wrappedAction = withMutationDisabled(action);
+            Action<? super I> decoratedAction = getEventRegister().getDecorator().decorate(wrappedAction);
+
+            if (object != null) {
+                // Already realized, just run the action now
+                decoratedAction.execute(object);
+                return;
+            }
+            // Collect any container level add actions then add the object specific action
+            onCreate = onCreate.mergeFrom(getEventRegister().getAddActions()).add(decoratedAction);
+        }
+
+        @Override
+        public I get() {
+            if (wasElementRemoved()) {
+                throw domainObjectRemovedException(getName(), getType());
+            }
+            return super.get();
+        }
+
+        @Override
+        public I getOrNull() {
+            if (wasElementRemoved()) {
+                return null;
+            }
+            if (failure != null) {
+                throw failure;
+            }
+            if (object == null) {
+                object = getType().cast(findByNameWithoutRules(getName()));
+                if (object == null) {
+                    tryCreate();
+                }
+            }
+            return object;
+        }
+
+        protected void tryCreate() {
+            try {
+                // Collect any container level add actions added since the last call to configure()
+                onCreate = onCreate.mergeFrom(getEventRegister().getAddActions());
+
+                // Create the domain object
+                object = createDomainObject();
+
+                // Register the domain object
+                add(object, onCreate);
+                realized(AbstractDomainObjectCreatingProvider.this);
+                onLazyDomainObjectRealized();
+            } catch (Throwable ex) {
+                failure = domainObjectCreationException(ex);
+                throw failure;
+            } finally {
+                // Discard state that is no longer required
+                onCreate = ImmutableActionSet.empty();
+            }
+        }
+
+        protected abstract I createDomainObject();
+
+        protected void onLazyDomainObjectRealized() {
+            // Do nothing.
+        }
+
+        protected boolean wasElementRemoved() {
+            // Check for presence as the domain object may have been replaced
+            return (wasElementRemovedBeforeRealized() || wasElementRemovedAfterRealized()) && !isPresent();
+        }
+
+        private boolean wasElementRemovedBeforeRealized() {
+            return removedBeforeRealized;
+        }
+
+        private boolean wasElementRemovedAfterRealized() {
+            return object != null && findByNameWithoutRules(getName()) == null;
+        }
+
+        protected RuntimeException domainObjectCreationException(Throwable cause) {
+            return new IllegalStateException(String.format("Could not create domain object '%s' (%s)", getName(), getType().getSimpleName()), cause);
+        }
+    }
+
+    private static RuntimeException domainObjectRemovedException(String name, Class<?> type) {
+        return new IllegalStateException(String.format("The domain object '%s' (%s) for this provider is no longer present in its container.", name, type.getSimpleName()));
+    }
 }
