@@ -27,11 +27,13 @@ import org.gradle.api.file.DeleteSpec;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.initialization.dsl.ScriptHandler;
 import org.gradle.api.internal.ProcessOperations;
+import org.gradle.api.internal.file.DefaultFileCollectionFactory;
 import org.gradle.api.internal.file.DefaultFileOperations;
+import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FileLookup;
 import org.gradle.api.internal.file.FileOperations;
 import org.gradle.api.internal.file.FileResolver;
-import org.gradle.api.internal.file.HasFileOperations;
+import org.gradle.api.internal.file.HasScriptServices;
 import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.api.internal.initialization.ScriptHandlerFactory;
@@ -39,7 +41,6 @@ import org.gradle.api.internal.plugins.DefaultObjectConfigurationAction;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
 import org.gradle.api.logging.LoggingManager;
-import org.gradle.api.provider.PropertyState;
 import org.gradle.api.provider.Provider;
 import org.gradle.api.provider.ProviderFactory;
 import org.gradle.api.resources.ResourceHandler;
@@ -48,9 +49,11 @@ import org.gradle.configuration.ScriptPluginFactory;
 import org.gradle.internal.Actions;
 import org.gradle.internal.hash.FileHasher;
 import org.gradle.internal.hash.StreamHasher;
+import org.gradle.internal.nativeintegration.filesystem.FileSystem;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.resource.TextResourceLoader;
 import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.time.Clock;
 import org.gradle.process.ExecResult;
 import org.gradle.process.ExecSpec;
 import org.gradle.process.JavaExecSpec;
@@ -77,25 +80,32 @@ public abstract class DefaultScript extends BasicScript {
         super.init(target, services);
         this.__scriptServices = services;
         loggingManager = services.get(LoggingManager.class);
-        Instantiator instantiator = services.get(Instantiator.class);
-        FileLookup fileLookup = services.get(FileLookup.class);
-        ExecFactory execFactory = services.get(ExecFactory.class);
-        DirectoryFileTreeFactory directoryFileTreeFactory = services.get(DirectoryFileTreeFactory.class);
-        StreamHasher streamHasher = services.get(StreamHasher.class);
-        FileHasher fileHasher = services.get(FileHasher.class);
-        TextResourceLoader textResourceLoader = services.get(TextResourceLoader.class);
-        if (target instanceof HasFileOperations) {
-            fileOperations = ((HasFileOperations) target).getFileOperations();
+        if (target instanceof HasScriptServices) {
+            HasScriptServices scriptServices = (HasScriptServices) target;
+            fileOperations = scriptServices.getFileOperations();
+            processOperations = scriptServices.getProcessOperations();
         } else {
+            Instantiator instantiator = services.get(Instantiator.class);
+            FileLookup fileLookup = services.get(FileLookup.class);
+            FileSystem fileSystem = services.get(FileSystem.class);
+            Clock clock = services.get(Clock.class);
+            DirectoryFileTreeFactory directoryFileTreeFactory = services.get(DirectoryFileTreeFactory.class);
+            StreamHasher streamHasher = services.get(StreamHasher.class);
+            FileHasher fileHasher = services.get(FileHasher.class);
+            TextResourceLoader textResourceLoader = services.get(TextResourceLoader.class);
+            FileCollectionFactory fileCollectionFactory = services.get(FileCollectionFactory.class);
             File sourceFile = getScriptSource().getResource().getLocation().getFile();
             if (sourceFile != null) {
-                fileOperations = new DefaultFileOperations(fileLookup.getFileResolver(sourceFile.getParentFile()), null, null, instantiator, fileLookup, directoryFileTreeFactory, streamHasher, fileHasher, execFactory, textResourceLoader);
+                FileResolver resolver = fileLookup.getFileResolver(sourceFile.getParentFile());
+                DefaultFileCollectionFactory fileCollectionFactoryWithBase = new DefaultFileCollectionFactory(resolver, null);
+                fileOperations = new DefaultFileOperations(resolver, null, null, instantiator, fileLookup, directoryFileTreeFactory, streamHasher, fileHasher, textResourceLoader, fileCollectionFactoryWithBase, fileSystem, clock);
+                processOperations = services.get(ExecFactory.class).forContext(resolver, fileCollectionFactoryWithBase, instantiator);
             } else {
-                fileOperations = new DefaultFileOperations(fileLookup.getFileResolver(), null, null, instantiator, fileLookup, directoryFileTreeFactory, streamHasher, fileHasher, execFactory, textResourceLoader);
+                fileOperations = new DefaultFileOperations(fileLookup.getFileResolver(), null, null, instantiator, fileLookup, directoryFileTreeFactory, streamHasher, fileHasher, textResourceLoader, fileCollectionFactory, fileSystem, clock);
+                processOperations = services.get(ExecFactory.class);
             }
         }
 
-        processOperations = (ProcessOperations) fileOperations;
         providerFactory = services.get(ProviderFactory.class);
     }
 
@@ -103,11 +113,6 @@ public abstract class DefaultScript extends BasicScript {
         return fileOperations.getFileResolver();
     }
 
-    @Override
-    public FileOperations getFileOperations() {
-        return fileOperations;
-    }
-
     private DefaultObjectConfigurationAction createObjectConfigurationAction() {
         ClassLoaderScope classLoaderScope = __scriptServices.get(ClassLoaderScope.class);
         return new DefaultObjectConfigurationAction(
@@ -166,7 +171,7 @@ public abstract class DefaultScript extends BasicScript {
 
     @Override
     public ConfigurableFileCollection files(Object paths, Closure configureClosure) {
-        return ConfigureUtil.configure(configureClosure, fileOperations.configurableFiles(paths));
+        return ConfigureUtil.configure(configureClosure, files(paths));
     }
 
     @Override
@@ -265,11 +270,6 @@ public abstract class DefaultScript extends BasicScript {
         return providerFactory.provider(value);
     }
 
-    @Override
-    public <T> PropertyState<T> property(Class<T> clazz) {
-        return providerFactory.property(clazz);
-    }
-
     @Override
     public LoggingManager getLogging() {
         return loggingManager;
