@@ -22,10 +22,10 @@ import org.gradle.api.Action;
 import org.gradle.api.internal.initialization.loadercache.ClassLoaderCache;
 import org.gradle.api.internal.initialization.loadercache.ClassLoaderId;
 import org.gradle.cache.CacheRepository;
-import org.gradle.cache.CacheValidator;
 import org.gradle.cache.PersistentCache;
 import org.gradle.groovy.scripts.ScriptSource;
 import org.gradle.internal.UncheckedException;
+import org.gradle.internal.classanalysis.AsmConstants;
 import org.gradle.internal.classloader.ClassLoaderHierarchyHasher;
 import org.gradle.internal.hash.HashCode;
 import org.gradle.internal.hash.HashUtil;
@@ -52,22 +52,19 @@ import java.net.URI;
  * A {@link ScriptClassCompiler} which compiles scripts to a cache directory, and loads them from there.
  */
 public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler, Closeable {
+    private static final Object[] EMPTY_OBJECT_ARRAY = new Object[0];
     private final ScriptCompilationHandler scriptCompilationHandler;
     private final ProgressLoggerFactory progressLoggerFactory;
     private final CacheRepository cacheRepository;
-    private final CacheValidator validator;
-    private final ScriptSourceHasher hasher;
     private final ClassLoaderCache classLoaderCache;
     private final ClassLoaderHierarchyHasher classLoaderHierarchyHasher;
 
-    public FileCacheBackedScriptClassCompiler(CacheRepository cacheRepository, CacheValidator validator, ScriptCompilationHandler scriptCompilationHandler,
-                                              ProgressLoggerFactory progressLoggerFactory, ScriptSourceHasher hasher, ClassLoaderCache classLoaderCache,
+    public FileCacheBackedScriptClassCompiler(CacheRepository cacheRepository, ScriptCompilationHandler scriptCompilationHandler,
+                                              ProgressLoggerFactory progressLoggerFactory, ClassLoaderCache classLoaderCache,
                                               ClassLoaderHierarchyHasher classLoaderHierarchyHasher) {
         this.cacheRepository = cacheRepository;
-        this.validator = validator;
         this.scriptCompilationHandler = scriptCompilationHandler;
         this.progressLoggerFactory = progressLoggerFactory;
-        this.hasher = hasher;
         this.classLoaderCache = classLoaderCache;
         this.classLoaderHierarchyHasher = classLoaderHierarchyHasher;
     }
@@ -84,7 +81,7 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             return emptyCompiledScript(classLoaderId, operation);
         }
 
-        HashCode sourceHashCode = hasher.hash(source);
+        HashCode sourceHashCode = source.getResource().getContentHash();
         final String sourceHash = HashUtil.compactStringFor(sourceHashCode);
         final String dslId = operation.getId();
         HashCode classLoaderHash = classLoaderHierarchyHasher.getClassLoaderHash(classLoader);
@@ -102,7 +99,6 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         // For 2, if the script changes, a different cache is used. If the classpath changes, the cache is invalidated, but classes are remapped to 1. anyway so never directly used
         PersistentCache remappedClassesCache = cacheRepository.cache("scripts-remapped/" + source.getClassName() + "/" + sourceHash + "/" + classpathHash)
             .withDisplayName(dslId + " remapped class cache for " + sourceHash)
-            .withValidator(validator)
             .withInitializer(new ProgressReportingInitializer(progressLoggerFactory, new RemapBuildScriptsAction<M, T>(remapped, classpathHash, sourceHash, dslId, classLoader, operation, verifier, scriptBaseClass),
                 "Compiling script into cache",
                 "Compiling " + source.getFileName() + " into local compilation cache"))
@@ -217,7 +213,7 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         private final String contentHash;
 
         public BuildScriptRemapper(ClassVisitor cv, ScriptSource source, String originalClassName, String contentHash) {
-            super(ASM6, cv);
+            super(AsmConstants.ASM_LEVEL, cv);
             this.scriptSource = source;
             this.originalClassName = originalClassName;
             this.contentHash = contentHash;
@@ -292,6 +288,13 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             if (name == null) {
                 return null;
             }
+            if (RuleVisitor.SOURCE_URI_TOKEN.equals(name)) {
+                URI uri = scriptSource.getResource().getLocation().getURI();
+                return uri == null ? null : uri.toString();
+            }
+            if (RuleVisitor.SOURCE_DESC_TOKEN.equals(name)) {
+                return scriptSource.getDisplayName();
+            }
             return name.replaceAll(RemappingScriptSource.MAPPED_SCRIPT, scriptSource.getClassName());
         }
 
@@ -299,16 +302,23 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             if (o instanceof Type) {
                 return Type.getType(remap(((Type) o).getDescriptor()));
             }
-            if (RuleVisitor.SOURCE_URI_TOKEN.equals(o)) {
-                URI uri = scriptSource.getResource().getLocation().getURI();
-                return uri == null ? null : uri.toString();
-            }
-            if (RuleVisitor.SOURCE_DESC_TOKEN.equals(o)) {
-                return scriptSource.getDisplayName();
+            if (o instanceof String) {
+                return remap((String) o);
             }
             return o;
         }
 
+        private Object[] remap(int count, Object[] original) {
+            if (count == 0) {
+                return EMPTY_OBJECT_ARRAY;
+            }
+            Object[] remapped = new Object[count];
+            for (int idx = 0; idx < count; idx++) {
+                remapped[idx] = remap(original[idx]);
+            }
+            return remapped;
+        }
+
         public MethodVisitor visitMethod(int access, String name, String desc, String signature, String[] exceptions) {
             MethodVisitor mv = cv.visitMethod(access, name, remap(desc), remap(signature), remap(exceptions));
             if (mv != null && (access & ACC_ABSTRACT) == 0) {
@@ -340,7 +350,7 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         class MethodRenamer extends MethodVisitor {
 
             public MethodRenamer(final MethodVisitor mv) {
-                super(ASM6, mv);
+                super(AsmConstants.ASM_LEVEL, mv);
             }
 
             public void visitTypeInsn(int i, String name) {
@@ -370,8 +380,11 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
                 return super.visitAnnotation(remap(desc), visible);
             }
 
+            @Override
+            public void visitFrame(int type, int nLocal, Object[] local, int nStack, Object[] stack) {
+                super.visitFrame(type, nLocal, remap(nLocal, local), nStack, remap(nStack, stack));
+            }
         }
-
     }
 
     private class RemapBuildScriptsAction<M, T extends Script> implements Action<PersistentCache> {
@@ -399,7 +412,6 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
 
         public void execute(final PersistentCache remappedClassesCache) {
             final PersistentCache cache = cacheRepository.cache("scripts/" + sourceHash + "/" + dslId + "/" + classpathHash)
-                .withValidator(validator)
                 .withDisplayName(dslId + " generic class cache for " + source.getDisplayName())
                 .withInitializer(new ProgressReportingInitializer(
                     progressLoggerFactory,
@@ -435,7 +447,7 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
                         byte[] contents = Files.toByteArray(file);
                         ClassReader cr = new ClassReader(contents);
                         String originalClassName = cr.getClassName();
-                        String contentHash = Hashing.md5().hashBytes(contents).toString();
+                        String contentHash = Hashing.hashBytes(contents).toString();
                         BuildScriptRemapper remapper = new BuildScriptRemapper(cv, origin, originalClassName, contentHash);
                         cr.accept(remapper, 0);
                         Files.write(cv.toByteArray(), new File(relocalizedDir, renamed));
