@@ -17,6 +17,7 @@ package org.gradle.initialization.layout;
 
 import org.gradle.initialization.SettingsLocation;
 
+import javax.annotation.Nullable;
 import java.io.File;
 
 public class BuildLayout extends SettingsLocation {
@@ -24,7 +25,7 @@ public class BuildLayout extends SettingsLocation {
 
     // Note: `null` for `settingsFile` means explicitly no settings
     //       A non null value can be a non existent file, which is semantically equivalent to an empty file
-    public BuildLayout(File rootDirectory, File settingsDir, File settingsFile) {
+    public BuildLayout(File rootDirectory, File settingsDir, @Nullable File settingsFile) {
         super(settingsDir, settingsFile);
         this.rootDirectory = rootDirectory;
     }
