@@ -16,10 +16,11 @@
 
 package org.gradle.internal.artifacts.repositories;
 
-import org.gradle.api.Incubating;
 import org.gradle.api.artifacts.repositories.AuthenticationSupported;
+import org.gradle.api.credentials.Credentials;
 import org.gradle.authentication.Authentication;
 
+import javax.annotation.Nullable;
 import java.util.Collection;
 
 public interface AuthenticationSupportedInternal extends AuthenticationSupported {
@@ -27,6 +28,10 @@ public interface AuthenticationSupportedInternal extends AuthenticationSupported
      * Returns the configured authentication schemes or an instance of {@link org.gradle.internal.authentication.AllSchemesAuthentication}
      * if none have been configured yet credentials have been configured.
      */
-    @Incubating
     Collection<Authentication> getConfiguredAuthentication();
+
+    void setConfiguredCredentials(Credentials credentials);
+
+    @Nullable
+    Credentials getConfiguredCredentials();
 }
