@@ -16,15 +16,23 @@
 
 package org.gradle.process.internal;
 
+import org.gradle.api.internal.ProcessOperations;
+import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FileResolver;
+import org.gradle.initialization.BuildCancellationToken;
 import org.gradle.internal.reflect.Instantiator;
 
 /**
  * Manages forking/spawning processes.
  */
-public interface ExecFactory extends ExecActionFactory, ExecHandleFactory, JavaExecHandleFactory, DslExecActionFactory {
+public interface ExecFactory extends ExecActionFactory, ExecHandleFactory, JavaExecHandleFactory, JavaForkOptionsFactory, DslExecActionFactory, ProcessOperations {
     /**
      * Creates a new factory for the given context.
      */
-    ExecFactory forContext(FileResolver fileResolver, Instantiator instantiator);
+    ExecFactory forContext(FileResolver fileResolver, FileCollectionFactory fileCollectionFactory, Instantiator instantiator);
+
+    /**
+     * Creates a new factory for the given context.
+     */
+    ExecFactory forContext(FileResolver fileResolver, FileCollectionFactory fileCollectionFactory, Instantiator instantiator, BuildCancellationToken buildCancellationToken);
 }
