@@ -16,9 +16,11 @@
 package org.gradle.process.internal;
 
 import com.google.common.collect.Iterables;
+import org.gradle.api.file.ConfigurableFileCollection;
 import org.gradle.api.file.FileCollection;
+import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FileResolver;
-import org.gradle.api.internal.file.collections.DefaultConfigurableFileCollection;
+import org.gradle.initialization.BuildCancellationToken;
 import org.gradle.process.CommandLineArgumentProvider;
 import org.gradle.process.JavaExecSpec;
 import org.gradle.process.JavaForkOptions;
@@ -36,26 +38,25 @@ import java.util.concurrent.Executor;
  * Use {@link JavaExecHandleFactory} instead.
  */
 public class JavaExecHandleBuilder extends AbstractExecHandleBuilder implements JavaExecSpec {
+    private final FileCollectionFactory fileCollectionFactory;
     private String mainClass;
     private final List<Object> applicationArgs = new ArrayList<Object>();
-    private FileCollection classpath;
+    private ConfigurableFileCollection classpath;
     private final JavaForkOptions javaOptions;
-    private final FileResolver fileResolver;
     private final List<CommandLineArgumentProvider> argumentProviders = new ArrayList<CommandLineArgumentProvider>();
 
-    public JavaExecHandleBuilder(FileResolver fileResolver, Executor executor) {
-        super(fileResolver, executor);
-        this.fileResolver = fileResolver;
-        javaOptions = new DefaultJavaForkOptions(fileResolver);
-        classpath = new DefaultConfigurableFileCollection(fileResolver, null);
+    public JavaExecHandleBuilder(FileResolver fileResolver, FileCollectionFactory fileCollectionFactory, Executor executor, BuildCancellationToken buildCancellationToken) {
+        super(fileResolver, executor, buildCancellationToken);
+        this.fileCollectionFactory = fileCollectionFactory;
+        javaOptions = new DefaultJavaForkOptions(fileResolver, fileCollectionFactory);
         executable(javaOptions.getExecutable());
     }
 
     public List<String> getAllJvmArgs() {
         List<String> allArgs = new ArrayList<String>(javaOptions.getAllJvmArgs());
-        if (!classpath.isEmpty()) {
+        if (classpath != null && !classpath.isEmpty()) {
             allArgs.add("-cp");
-            allArgs.add(CollectionUtils.join(File.pathSeparator, classpath.getFiles()));
+            allArgs.add(CollectionUtils.join(File.pathSeparator, classpath));
         }
         return allArgs;
     }
@@ -206,16 +207,25 @@ public class JavaExecHandleBuilder extends AbstractExecHandleBuilder implements
     }
 
     public JavaExecHandleBuilder setClasspath(FileCollection classpath) {
-        this.classpath = classpath;
+        ConfigurableFileCollection newClasspath = fileCollectionFactory.configurableFiles("classpath");
+        newClasspath.setFrom(classpath);
+        this.classpath = newClasspath;
         return this;
     }
 
     public JavaExecHandleBuilder classpath(Object... paths) {
-        classpath = classpath.plus(fileResolver.resolveFiles(paths));
+        doGetClasspath().from(paths);
         return this;
     }
 
     public FileCollection getClasspath() {
+        return doGetClasspath();
+    }
+
+    private ConfigurableFileCollection doGetClasspath() {
+        if (classpath == null) {
+            classpath = fileCollectionFactory.configurableFiles("classpath");
+        }
         return classpath;
     }
 
