@@ -16,8 +16,10 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder;
 
 import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
+import com.google.common.collect.Sets;
 import org.gradle.api.Action;
 import org.gradle.api.GradleException;
 import org.gradle.api.artifacts.Configuration;
@@ -25,6 +27,7 @@ import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
+import org.gradle.api.attributes.Attribute;
 import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ComponentSelectorConverter;
 import org.gradle.api.internal.artifacts.ResolveContext;
@@ -41,9 +44,13 @@ import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflict
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts.ModuleConflictHandler;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts.PotentialConflict;
 import org.gradle.api.internal.attributes.AttributesSchemaInternal;
+import org.gradle.api.internal.attributes.CompatibilityRule;
+import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
 import org.gradle.api.specs.Spec;
+import org.gradle.internal.component.IncompatibleVariantsSelectionException;
 import org.gradle.internal.component.external.model.DefaultModuleComponentIdentifier;
+import org.gradle.internal.component.model.DefaultCompatibilityCheckResult;
 import org.gradle.internal.component.model.DependencyMetadata;
 import org.gradle.internal.id.IdGenerator;
 import org.gradle.internal.id.LongIdGenerator;
@@ -61,9 +68,12 @@ import org.slf4j.LoggerFactory;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 public class DependencyGraphBuilder {
     private static final Logger LOGGER = LoggerFactory.getLogger(DependencyGraphBuilder.class);
@@ -196,7 +206,8 @@ public class DependencyGraphBuilder {
                         for (ComponentState version : versions) {
                             List<NodeState> nodes = version.getNodes();
                             for (NodeState nodeState : nodes) {
-                                if (nodeState.isSelected() && doesNotDeclareExplicitCapability(nodeState)) {
+                                // Collect nodes as implicit capability providers if different than current node, selected and not having explicit capabilities
+                                if (node != nodeState && nodeState.isSelected() && doesNotDeclareExplicitCapability(nodeState)) {
                                     implicitProvidersForCapability.add(nodeState);
                                 }
                             }
@@ -334,18 +345,83 @@ public class DependencyGraphBuilder {
 
     private void validateGraph(ResolveState resolveState) {
         for (ModuleResolveState module : resolveState.getModules()) {
-            if (module.getSelected() != null && module.getSelected().isRejected()) {
-                GradleException error = new GradleException(new RejectedModuleMessageBuilder().buildFailureMessage(module));
-                attachFailureToEdges(error, module.getIncomingEdges());
-                // We need to attach failures on unattached dependencies too, in case a node wasn't selected
-                // at all, but we still want to see an error message for it.
-                attachFailureToEdges(error, module.getUnattachedDependencies());
+            ComponentState selected = module.getSelected();
+            if (selected != null) {
+                if (selected.isRejected()) {
+                    GradleException error = new GradleException(selected.getRejectedErrorMessage());
+                    attachFailureToEdges(error, module.getIncomingEdges());
+                    // We need to attach failures on unattached dependencies too, in case a node wasn't selected
+                    // at all, but we still want to see an error message for it.
+                    attachFailureToEdges(error, module.getUnattachedDependencies());
+                } else {
+                    if (module.isVirtualPlatform()) {
+                        attachMultipleForceOnPlatformFailureToEdges(module);
+                    } else if (selected.hasMoreThanOneSelectedNodeUsingVariantAwareResolution()) {
+                        validateMultipleNodeSelection(module, selected);
+                    }
+                }
             } else if (module.isVirtualPlatform()) {
                 attachMultipleForceOnPlatformFailureToEdges(module);
             }
         }
     }
 
+    /**
+     * Validates that all selected nodes of a single component have compatible attributes,
+     * when using variant aware resolution.
+     */
+    private void validateMultipleNodeSelection(ModuleResolveState module, ComponentState selected) {
+        Set<NodeState> selectedNodes = selected.getNodes().stream()
+                .filter(n -> n.isSelected() && !n.isAttachedToVirtualPlatform() && !n.hasShadowedCapability())
+                .collect(Collectors.toSet());
+        if (selectedNodes.size()<2) {
+            return;
+        }
+        Set<Set<NodeState>> combinations = Sets.combinations(selectedNodes, 2);
+        Set<NodeState> incompatibleNodes = Sets.newHashSet();
+        for (Set<NodeState> combination : combinations) {
+            Iterator<NodeState> it = combination.iterator();
+            NodeState first = it.next();
+            NodeState second = it.next();
+            assertCompatibleAttributes(first, second, incompatibleNodes);
+        }
+        if (!incompatibleNodes.isEmpty()) {
+            IncompatibleVariantsSelectionException variantsSelectionException = new IncompatibleVariantsSelectionException(
+                    IncompatibleVariantsSelectionMessageBuilder.buildMessage(selected, incompatibleNodes)
+            );
+            for (EdgeState edge : module.getIncomingEdges()) {
+                edge.failWith(variantsSelectionException);
+            }
+        }
+    }
+
+    private void assertCompatibleAttributes(NodeState first, NodeState second, Set<NodeState> incompatibleNodes) {
+        ImmutableAttributes firstAttributes = first.getMetadata().getAttributes();
+        ImmutableAttributes secondAttributes = second.getMetadata().getAttributes();
+        ImmutableSet<Attribute<?>> firstKeys = firstAttributes.keySet();
+        ImmutableSet<Attribute<?>> secondKeys = secondAttributes.keySet();
+        for (Attribute<?> attribute : Sets.intersection(firstKeys, secondKeys)) {
+            CompatibilityRule<Object> rule = attributesSchema.compatibilityRules(attribute);
+            Object v1 = firstAttributes.getAttribute(attribute);
+            Object v2 = secondAttributes.getAttribute(attribute);
+            // for all commons attributes, make sure they are compatible with each other
+            if (!compatible(rule, v1, v2) || !compatible(rule, v2, v1)) {
+                incompatibleNodes.add(first);
+                incompatibleNodes.add(second);
+            }
+        }
+    }
+
+    private static boolean compatible(CompatibilityRule<Object> rule, Object v1, Object v2) {
+        if (Objects.equals(v1, v2)) {
+            // Equal values are compatible
+            return true;
+        }
+        DefaultCompatibilityCheckResult<Object> result = new DefaultCompatibilityCheckResult<>(v1, v2);
+        rule.execute(result);
+        return result.hasResult() && result.isCompatible();
+    }
+
     private void attachMultipleForceOnPlatformFailureToEdges(ModuleResolveState module) {
         List<EdgeState> forcedEdges = null;
         boolean hasMultipleVersions = false;
