@@ -23,8 +23,10 @@ import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.result.ComponentSelectionReason;
+import org.gradle.api.attributes.Attribute;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusion;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphEdge;
+import org.gradle.api.internal.attributes.AttributeMergingException;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.component.local.model.DslOriginDependencyMetadata;
 import org.gradle.internal.component.model.ComponentArtifactMetadata;
@@ -58,6 +60,7 @@ class EdgeState implements DependencyGraphEdge {
     private final boolean isConstraint;
 
     private ModuleVersionResolveException targetNodeSelectionFailure;
+    private ImmutableAttributes cachedAttributes;
 
     EdgeState(NodeState from, DependencyState dependencyState, ModuleExclusion transitiveExclusions, ResolveState resolveState) {
         this.from = from;
@@ -165,28 +168,48 @@ class EdgeState implements DependencyGraphEdge {
     }
 
     public ImmutableAttributes getAttributes() {
+        assert cachedAttributes != null;
+        return cachedAttributes;
+    }
+
+    private ImmutableAttributes safeGetAttributes() throws AttributeMergingException {
         ModuleResolveState module = selector.getTargetModule();
-        return module.mergedConstraintsAttributes(dependencyState.getRequested().getAttributes());
+        cachedAttributes = module.mergedConstraintsAttributes(dependencyState.getRequested().getAttributes());
+        return cachedAttributes;
     }
 
     private void calculateTargetConfigurations(ComponentState targetComponent) {
+        ComponentResolveMetadata targetModuleVersion = targetComponent.getMetadata();
         targetNodes.clear();
         targetNodeSelectionFailure = null;
-        ComponentResolveMetadata targetModuleVersion = targetComponent.getMetadata();
         if (targetModuleVersion == null) {
             targetComponent.getModule().getPlatformState().addOrphanEdge(this);
             // Broken version
             return;
         }
+        if (isConstraint && !isVirtualDependency()) {
+            List<NodeState> nodes = targetComponent.getNodes();
+            for (NodeState node : nodes) {
+                if (node.isSelected()) {
+                    targetNodes.add(node);
+                }
+            }
+            return;
+        }
 
         List<ConfigurationMetadata> targetConfigurations;
         try {
             ImmutableAttributes attributes = resolveState.getRoot().getMetadata().getAttributes();
-            attributes = resolveState.getAttributesFactory().concat(attributes, getAttributes());
-            if (isConstraint) {
-                attributes = selector.getTargetModule().mergeConstraintAttributesWithHardDependencyAttributes(attributes);
-            }
-            targetConfigurations = dependencyMetadata.selectConfigurations(attributes, targetModuleVersion, resolveState.getAttributesSchema());
+            attributes = resolveState.getAttributesFactory().concat(attributes, safeGetAttributes());
+            targetConfigurations = dependencyMetadata.selectConfigurations(attributes, targetModuleVersion, resolveState.getAttributesSchema(), dependencyState.getRequested().getRequestedCapabilities());
+        } catch (AttributeMergingException mergeError) {
+            targetNodeSelectionFailure = new ModuleVersionResolveException(dependencyState.getRequested(), () -> {
+                Attribute<?> attribute = mergeError.getAttribute();
+                Object constraintValue = mergeError.getLeftValue();
+                Object dependencyValue = mergeError.getRightValue();
+                return "Inconsistency between attributes of a constraint and a dependency, on attribute '" + attribute + "' : dependency requires '" + dependencyValue + "' while constraint required '" + constraintValue + "'";
+            });
+            return;
         } catch (Exception t) {
             // Failure to select the target variant/configurations from this component, given the dependency attributes/metadata.
             targetNodeSelectionFailure = new ModuleVersionResolveException(dependencyState.getRequested(), t);
@@ -198,6 +221,10 @@ class EdgeState implements DependencyGraphEdge {
         }
     }
 
+    private boolean isVirtualDependency() {
+        return selector.getDependencyMetadata() instanceof LenientPlatformDependencyMetadata;
+    }
+
     @Override
     public ModuleExclusion getExclusions() {
         List<ExcludeMetadata> excludes = dependencyMetadata.getExcludes();
@@ -281,9 +308,9 @@ class EdgeState implements DependencyGraphEdge {
         });
     }
 
-    void maybeDecreaseHardEdgeCount() {
+    void maybeDecreaseHardEdgeCount(NodeState removalSource) {
         if (!isConstraint) {
-            selector.getTargetModule().decreaseHardEdgeCount();
+            selector.getTargetModule().decreaseHardEdgeCount(removalSource);
         }
     }
 }
