@@ -34,6 +34,7 @@ import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.Dependen
 import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
 import org.gradle.api.specs.Spec;
 import org.gradle.internal.component.external.model.DefaultModuleComponentSelector;
+import org.gradle.internal.component.external.model.ShadowedCapability;
 import org.gradle.internal.component.local.model.LocalConfigurationMetadata;
 import org.gradle.internal.component.local.model.LocalFileDependencyMetadata;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
@@ -486,7 +487,7 @@ public class NodeState implements DependencyGraphNode {
             for (EdgeState outgoingDependency : outgoingEdges) {
                 outgoingDependency.removeFromTargetConfigurations();
                 outgoingDependency.getSelector().release();
-                outgoingDependency.maybeDecreaseHardEdgeCount();
+                outgoingDependency.maybeDecreaseHardEdgeCount(this);
             }
         }
         outgoingEdges.clear();
@@ -556,12 +557,16 @@ public class NodeState implements DependencyGraphNode {
      * Invoked when this node is back to being a pending dependency.
      * There may be some incoming edges left at that point, but they must all be coming from constraints.
      */
-    public void clearConstraintEdges(PendingDependencies pendingDependencies) {
+    public void clearConstraintEdges(PendingDependencies pendingDependencies, NodeState backToPendingSource) {
         for (EdgeState incomingEdge : incomingEdges) {
             assert incomingEdge.getDependencyMetadata().isConstraint();
-            incomingEdge.getSelector().release();
             NodeState from = incomingEdge.getFrom();
-            from.getOutgoingEdges().remove(incomingEdge);
+            if (from != backToPendingSource) {
+                // Only remove edges that come from a different node than the source of the dependency going back to pending
+                // The edges from the "From" will be removed first
+                incomingEdge.getSelector().release();
+                from.getOutgoingEdges().remove(incomingEdge);
+            }
             pendingDependencies.addNode(from);
         }
         incomingEdges.clear();
@@ -612,6 +617,15 @@ public class NodeState implements DependencyGraphNode {
         return false;
     }
 
+    boolean hasShadowedCapability() {
+        for (Capability capability : metaData.getCapabilities().getCapabilities()) {
+            if (capability instanceof ShadowedCapability) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     boolean isSelectedByVariantAwareResolution() {
         // the order is strange logically but here for performance optimization
         return selectedByVariantAwareResolution && isSelected();
