@@ -17,20 +17,16 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder
 
 import com.google.common.base.Joiner;
 import com.google.common.collect.Lists;
-import org.gradle.api.artifacts.ModuleIdentifier;
+import com.google.common.collect.Sets;
 import org.gradle.api.artifacts.result.ComponentSelectionDescriptor;
-import org.gradle.api.internal.artifacts.ResolvedVersionConstraint;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.CompositeVersionSelector;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.InverseVersionSelector;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasonInternal;
 
 import java.util.List;
 import java.util.Set;
 
-public class RejectedModuleMessageBuilder {
-    public String buildFailureMessage(ModuleResolveState module) {
+class RejectedModuleMessageBuilder {
+    String buildFailureMessage(ModuleResolveState module) {
         boolean hasRejectAll = false;
         for (SelectorState candidate : module.getSelectors()) {
             hasRejectAll |= candidate.getVersionConstraint().isRejectAll();
@@ -41,7 +37,11 @@ public class RejectedModuleMessageBuilder {
         } else {
             sb.append("Cannot find a version of '").append(module.getId()).append("' that satisfies the version constraints: \n");
         }
-        renderEdges(sb, module.getIncomingEdges());
+
+        Set<EdgeState> allEdges = Sets.newLinkedHashSet();
+        allEdges.addAll(module.getIncomingEdges());
+        allEdges.addAll(module.getUnattachedDependencies());
+        renderEdges(sb, allEdges);
         return sb.toString();
     }
 
@@ -58,56 +58,8 @@ public class RejectedModuleMessageBuilder {
         }
     }
 
-    static void renderSelector(StringBuilder sb, SelectorState selectorState) {
-        ResolvedVersionConstraint constraint = selectorState.getVersionConstraint();
-        ModuleIdentifier moduleId = selectorState.getTargetModule().getId();
-
-        sb.append('\'').append(moduleId.getGroup()).append(':').append(moduleId.getName());
-
-        if (constraint.isRejectAll()) {
-            sb.append("' rejects all versions");
-            return;
-        }
-
-        VersionSelector requiredSelector = constraint.getRequiredSelector();
-        VersionSelector rejectedSelector = constraint.getRejectedSelector();
-
-        if (rejectedSelector instanceof InverseVersionSelector) {
-            sb.append("' strictly '").append(requiredSelector.getSelector()).append("'");
-            return;
-        }
-
-        if (requiredSelector != null) {
-            sb.append(':').append(requiredSelector.getSelector());
-        }
-        sb.append("'");
-
-        if (constraint.getPreferredSelector() != null) {
-            sb.append(" prefers '").append(constraint.getPreferredSelector().getSelector()).append("'");
-        }
-
-        if (rejectedSelector == null) {
-            return;
-        }
-
-        sb.append(" rejects ");
-        if (rejectedSelector instanceof CompositeVersionSelector) {
-            sb.append("any of \"");
-            int i = 0;
-            for (VersionSelector selector : ((CompositeVersionSelector) rejectedSelector).getSelectors()) {
-                if (i++ > 0) {
-                    sb.append(", ");
-                }
-                sb.append('\'');
-                sb.append(selector.getSelector());
-                sb.append('\'');
-            }
-            sb.append("\"");
-        } else {
-            sb.append('\'');
-            sb.append(rejectedSelector.getSelector());
-            sb.append('\'');
-        }
+    private static void renderSelector(StringBuilder sb, SelectorState selectorState) {
+        sb.append('\'').append(selectorState.getRequested()).append('\'');
     }
 
     private static void renderReason(StringBuilder sb, SelectorState selector) {
