@@ -16,22 +16,28 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts;
 
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.ComponentResolutionState;
+import org.gradle.api.Action;
+import org.gradle.api.artifacts.ModuleIdentifier;
 
-class DefaultConflictResolutionResult implements ConflictResolutionResult {
-    private final PotentialConflict conflict;
-    private final ComponentResolutionState selected;
+import java.util.Collection;
 
-    public DefaultConflictResolutionResult(PotentialConflict conflict, ComponentResolutionState selected) {
-        this.conflict = conflict;
+class DefaultConflictResolutionResult<T> implements ConflictResolutionResult {
+    private final Collection<? extends ModuleIdentifier> participatingModules;
+    private final T selected;
+
+    public DefaultConflictResolutionResult(Collection<? extends ModuleIdentifier> participatingModules, T selected) {
+        this.participatingModules = participatingModules;
         this.selected = selected;
     }
 
-    public PotentialConflict getConflict() {
-        return conflict;
+    public void withParticipatingModules(Action<? super ModuleIdentifier> action) {
+        for (ModuleIdentifier module : participatingModules) {
+            action.execute(module);
+        }
     }
 
-    public ComponentResolutionState getSelected() {
+    public T getSelected() {
         return selected;
     }
+
 }
