@@ -17,65 +17,99 @@
 package org.gradle.api.internal.artifacts.transform;
 
 import com.google.common.collect.Lists;
+import com.google.common.reflect.TypeToken;
 import org.gradle.api.Action;
 import org.gradle.api.ActionConfiguration;
 import org.gradle.api.NonExtensible;
 import org.gradle.api.artifacts.transform.ArtifactTransform;
-import org.gradle.api.artifacts.transform.ArtifactTransformSpec;
 import org.gradle.api.artifacts.transform.TransformAction;
+import org.gradle.api.artifacts.transform.TransformParameters;
+import org.gradle.api.artifacts.transform.TransformSpec;
 import org.gradle.api.artifacts.transform.VariantTransform;
 import org.gradle.api.artifacts.transform.VariantTransformConfigurationException;
 import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.internal.DefaultActionConfiguration;
+import org.gradle.api.internal.artifacts.ArtifactTransformRegistration;
 import org.gradle.api.internal.artifacts.VariantTransformRegistry;
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
 import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
-import org.gradle.internal.classloader.ClassLoaderHierarchyHasher;
+import org.gradle.internal.Cast;
+import org.gradle.internal.instantiation.InstantiationScheme;
 import org.gradle.internal.instantiation.InstantiatorFactory;
-import org.gradle.internal.isolation.IsolatableFactory;
+import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.model.internal.type.ModelType;
 
 import javax.annotation.Nullable;
-import java.util.Collections;
+import java.lang.reflect.ParameterizedType;
+import java.util.Arrays;
 import java.util.List;
 
 public class DefaultVariantTransformRegistry implements VariantTransformRegistry {
     private static final Object[] NO_PARAMETERS = new Object[0];
-    private final List<Registration> transforms = Lists.newArrayList();
+    private final List<ArtifactTransformRegistration> transforms = Lists.newArrayList();
     private final ImmutableAttributesFactory immutableAttributesFactory;
-    private final IsolatableFactory isolatableFactory;
-    private final ClassLoaderHierarchyHasher classLoaderHierarchyHasher;
+    private final ServiceRegistry services;
     private final InstantiatorFactory instantiatorFactory;
-    private final TransformerInvoker transformerInvoker;
+    private final InstantiationScheme parametersInstantiationScheme;
+    private final TransformationRegistrationFactory registrationFactory;
 
-    public DefaultVariantTransformRegistry(InstantiatorFactory instantiatorFactory, ImmutableAttributesFactory immutableAttributesFactory, IsolatableFactory isolatableFactory, ClassLoaderHierarchyHasher classLoaderHierarchyHasher, TransformerInvoker transformerInvoker) {
+    public DefaultVariantTransformRegistry(InstantiatorFactory instantiatorFactory, ImmutableAttributesFactory immutableAttributesFactory, ServiceRegistry services, TransformationRegistrationFactory registrationFactory, InstantiationScheme parametersInstantiationScheme) {
         this.instantiatorFactory = instantiatorFactory;
         this.immutableAttributesFactory = immutableAttributesFactory;
-        this.isolatableFactory = isolatableFactory;
-        this.classLoaderHierarchyHasher = classLoaderHierarchyHasher;
-        this.transformerInvoker = transformerInvoker;
+        this.services = services;
+        this.registrationFactory = registrationFactory;
+        this.parametersInstantiationScheme = parametersInstantiationScheme;
     }
 
     @Override
     public void registerTransform(Action<? super VariantTransform> registrationAction) {
         UntypedRegistration registration = instantiatorFactory.decorateLenient().newInstance(UntypedRegistration.class, immutableAttributesFactory, instantiatorFactory);
-        register(registration, registrationAction);
+        registrationAction.execute(registration);
+
+        validateActionType(registration.actionType);
+        validateAttributes(registration);
+
+        Object[] parameters = registration.getTransformParameters();
+        try {
+            ArtifactTransformRegistration finalizedRegistration = registrationFactory.create(registration.from.asImmutable(), registration.to.asImmutable(), registration.actionType, parameters);
+            transforms.add(finalizedRegistration);
+        } catch (Exception e) {
+            throw new VariantTransformConfigurationException(String.format("Cannot register artifact transform %s with parameters %s", ModelType.of(registration.actionType).getDisplayName(), Arrays.toString(parameters)), e);
+        }
     }
 
     @Override
-    public <T> void registerTransform(Class<T> configurationType, Action<? super ArtifactTransformSpec<T>> registrationAction) {
-        // TODO - should inject services into configuration
-        // TODO - should decorate, need to stop using serialization of the config object
-        T configuration = instantiatorFactory.inject().newInstance(configurationType);
-        TypedRegistration<T> registration = instantiatorFactory.decorateLenient().newInstance(TypedRegistration.class, configuration, immutableAttributesFactory);
-        register(registration, registrationAction);
-    }
-
-    private <T extends RecordingRegistration> void register(T registration, Action<? super T> registrationAction) {
+    public <T extends TransformParameters> void registerTransform(Class<? extends TransformAction<T>> actionType, Action<? super TransformSpec<T>> registrationAction) {
+        ParameterizedType superType = (ParameterizedType) TypeToken.of(actionType).getSupertype(TransformAction.class).getType();
+        Class<T> parameterType = Cast.uncheckedNonnullCast(TypeToken.of(superType.getActualTypeArguments()[0]).getRawType());
+        if (parameterType == TransformParameters.class) {
+            throw new VariantTransformConfigurationException(String.format("Could not register transform: must use a sub-type of %s as parameter type. Use %s for transforms without parameters.", ModelType.of(TransformParameters.class).getDisplayName(), ModelType.of(TransformParameters.None.class).getDisplayName()));
+        }
+        T parameterObject = parameterType == TransformParameters.None.class ? null : parametersInstantiationScheme.withServices(services).newInstance(parameterType);
+        TypedRegistration<T> registration = Cast.uncheckedNonnullCast(instantiatorFactory.decorateLenient().newInstance(TypedRegistration.class, parameterObject, immutableAttributesFactory));
         registrationAction.execute(registration);
 
-        if (registration.actionType == null) {
-            throw new VariantTransformConfigurationException("Could not register transform: an ArtifactTransform must be provided.");
+        register(registration, actionType, parameterObject);
+    }
+
+    private <T extends TransformParameters> void register(RecordingRegistration registration, Class<? extends TransformAction> actionType, @Nullable T parameterObject) {
+        validateActionType(actionType);
+        validateAttributes(registration);
+        try {
+            ArtifactTransformRegistration finalizedRegistration = registrationFactory.create(registration.from.asImmutable(), registration.to.asImmutable(), actionType, parameterObject);
+            transforms.add(finalizedRegistration);
+        } catch (Exception e) {
+            throw new VariantTransformConfigurationException(String.format("Cannot register artifact transform %s with parameters %s", ModelType.of(actionType).getDisplayName(), parameterObject), e);
         }
+    }
+
+    private static <T> void validateActionType(@Nullable Class<T> actionType) {
+        if (actionType == null) {
+            throw new VariantTransformConfigurationException("Could not register transform: an artifact transform action must be provided.");
+        }
+    }
+
+    private static void validateAttributes(RecordingRegistration registration) {
         if (registration.to.isEmpty()) {
             throw new VariantTransformConfigurationException("Could not register transform: at least one 'to' attribute must be provided.");
         }
@@ -85,24 +119,15 @@ public class DefaultVariantTransformRegistry implements VariantTransformRegistry
         if (!registration.from.keySet().containsAll(registration.to.keySet())) {
             throw new VariantTransformConfigurationException("Could not register transform: each 'to' attribute must be included as a 'from' attribute.");
         }
-
-        // TODO - should calculate this lazily
-        Object[] parameters = registration.getTransformParameters();
-        Object config = registration.getConfig();
-
-        Registration finalizedRegistration = DefaultTransformationRegistration.create(registration.from.asImmutable(), registration.to.asImmutable(), registration.actionType, config, parameters, isolatableFactory, classLoaderHierarchyHasher, instantiatorFactory, transformerInvoker);
-        transforms.add(finalizedRegistration);
     }
 
-    public Iterable<Registration> getTransforms() {
+    public Iterable<ArtifactTransformRegistration> getTransforms() {
         return transforms;
     }
 
     public static abstract class RecordingRegistration {
         final AttributeContainerInternal from;
         final AttributeContainerInternal to;
-        Class<? extends ArtifactTransform> actionType;
-        Action<? super ActionConfiguration> configAction;
 
         public RecordingRegistration(ImmutableAttributesFactory immutableAttributesFactory) {
             from = immutableAttributesFactory.mutable();
@@ -116,16 +141,14 @@ public class DefaultVariantTransformRegistry implements VariantTransformRegistry
         public AttributeContainer getTo() {
             return to;
         }
-
-        abstract Object[] getTransformParameters();
-
-        @Nullable
-        abstract Object getConfig();
     }
 
+    @SuppressWarnings("deprecation")
     @NonExtensible
     public static class UntypedRegistration extends RecordingRegistration implements VariantTransform {
+        private Action<? super ActionConfiguration> configAction;
         private final InstantiatorFactory instantiatorFactory;
+        Class<? extends ArtifactTransform> actionType;
 
         public UntypedRegistration(ImmutableAttributesFactory immutableAttributesFactory, InstantiatorFactory instantiatorFactory) {
             super(immutableAttributesFactory);
@@ -146,7 +169,6 @@ public class DefaultVariantTransformRegistry implements VariantTransformRegistry
             this.configAction = config;
         }
 
-        @Override
         Object[] getTransformParameters() {
             if (configAction == null) {
                 return NO_PARAMETERS;
@@ -155,72 +177,31 @@ public class DefaultVariantTransformRegistry implements VariantTransformRegistry
             configAction.execute(config);
             return config.getParams();
         }
-
-        @Override
-        Object getConfig() {
-            return null;
-        }
     }
 
     @NonExtensible
-    public static class TypedRegistration<T> extends RecordingRegistration implements ArtifactTransformSpec<T> {
-        private final T config;
-        private final List<Object> params = Lists.newArrayList();
+    public static class TypedRegistration<T extends TransformParameters> extends RecordingRegistration implements TransformSpec<T> {
+        private final T parameterObject;
 
-        public TypedRegistration(T config, ImmutableAttributesFactory immutableAttributesFactory) {
+        public TypedRegistration(@Nullable T parameterObject, ImmutableAttributesFactory immutableAttributesFactory) {
             super(immutableAttributesFactory);
-            this.config = config;
-            TransformAction transformAction = config.getClass().getAnnotation(TransformAction.class);
-            if (transformAction != null) {
-                actionType = transformAction.value();
+            this.parameterObject = parameterObject;
+        }
+
+        @Override
+        public T getParameters() {
+            if (parameterObject == null) {
+                throw new VariantTransformConfigurationException("Cannot query parameters for artifact transform without parameters.");
             }
+            return parameterObject;
         }
 
         @Override
-        public Class<? extends ArtifactTransform> getActionClass() {
-            return actionType;
-        }
-
-        @Override
-        public void setActionClass(Class<? extends ArtifactTransform> implementationClass) {
-            this.actionType = implementationClass;
-        }
-
-        @Override
-        public T getConfiguration() {
-            return config;
-        }
-
-        @Override
-        public void configuration(Action<? super T> action) {
-            action.execute(config);
-        }
-
-        @Override
-        public Object[] getParams() {
-            return params.toArray();
-        }
-
-        @Override
-        public void setParams(Object[] params) {
-            this.params.clear();
-            Collections.addAll(this.params, params);
-        }
-
-        @Override
-        public void params(Object... params) {
-            Collections.addAll(this.params, params);
-        }
-
-        @Nullable
-        @Override
-        Object getConfig() {
-            return config;
-        }
-
-        @Override
-        Object[] getTransformParameters() {
-            return params.toArray();
+        public void parameters(Action<? super T> action) {
+            if (parameterObject == null) {
+                throw new VariantTransformConfigurationException("Cannot configure parameters for artifact transform without parameters.");
+            }
+            action.execute(parameterObject);
         }
     }
 }
