@@ -21,6 +21,7 @@ import org.gradle.api.Project;
 import org.gradle.api.artifacts.ResolveException;
 import org.gradle.api.internal.artifacts.ivyservice.DefaultLenientConfiguration;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvableArtifact;
+import org.gradle.execution.ProjectExecutionServiceRegistry;
 import org.gradle.execution.plan.Node;
 import org.gradle.execution.plan.TaskDependencyResolver;
 import org.gradle.internal.Try;
@@ -41,6 +42,7 @@ public abstract class TransformationNode extends Node {
 
     private final int order = ORDER_COUNTER.incrementAndGet();
     protected final TransformationStep transformationStep;
+    protected final ExecutionGraphDependenciesResolver dependenciesResolver;
     protected Try<TransformationSubject> transformedSubject;
 
     public static TransformationNode chained(TransformationStep current, TransformationNode previous, ExecutionGraphDependenciesResolver executionGraphDependenciesResolver) {
@@ -51,18 +53,29 @@ public abstract class TransformationNode extends Node {
         return new InitialTransformationNode(initial, artifact, executionGraphDependenciesResolver);
     }
 
-    protected TransformationNode(TransformationStep transformationStep) {
+    protected TransformationNode(TransformationStep transformationStep, ExecutionGraphDependenciesResolver dependenciesResolver) {
         this.transformationStep = transformationStep;
+        this.dependenciesResolver = dependenciesResolver;
     }
 
-    public abstract void execute(BuildOperationExecutor buildOperationExecutor, ArtifactTransformListener transformListener);
+    public abstract void execute(BuildOperationExecutor buildOperationExecutor, ArtifactTransformListener transformListener, ProjectExecutionServiceRegistry services);
+
+    @Override
+    public boolean isPublicNode() {
+        return true;
+    }
+
+    @Override
+    public boolean requiresMonitoring() {
+        return false;
+    }
 
     @Override
     public String toString() {
         return transformationStep.getDisplayName();
     }
 
-    private Try<TransformationSubject> getTransformedSubject() {
+    public Try<TransformationSubject> getTransformedSubject() {
         if (transformedSubject == null) {
             throw new IllegalStateException(String.format("Transformation %s has been scheduled and is now required, but did not execute, yet.", transformationStep.getDisplayName()));
         }
@@ -111,18 +124,24 @@ public abstract class TransformationNode extends Node {
         }
     }
 
+    @Override
+    public void resolveDependencies(TaskDependencyResolver dependencyResolver, Action<Node> processHardSuccessor) {
+        processDependencies(processHardSuccessor, dependencyResolver.resolveDependenciesFor(null, transformationStep.getDependencies()));
+        processDependencies(processHardSuccessor, dependencyResolver.resolveDependenciesFor(null, dependenciesResolver.computeDependencyNodes(transformationStep)));
+    }
+
     private static class InitialTransformationNode extends TransformationNode {
         private final ResolvableArtifact artifact;
         private final ExecutionGraphDependenciesResolver dependenciesResolver;
 
         public InitialTransformationNode(TransformationStep transformationStep, ResolvableArtifact artifact, ExecutionGraphDependenciesResolver dependenciesResolver) {
-            super(transformationStep);
+            super(transformationStep, dependenciesResolver);
             this.artifact = artifact;
             this.dependenciesResolver = dependenciesResolver;
         }
 
         @Override
-        public void execute(BuildOperationExecutor buildOperationExecutor, ArtifactTransformListener transformListener) {
+        public void execute(BuildOperationExecutor buildOperationExecutor, ArtifactTransformListener transformListener, ProjectExecutionServiceRegistry services) {
             this.transformedSubject = buildOperationExecutor.call(new ArtifactTransformationStepBuildOperation() {
                 @Override
                 protected Try<TransformationSubject> transform() {
@@ -137,7 +156,7 @@ public abstract class TransformationNode extends Node {
                     }
 
                     TransformationSubject initialArtifactTransformationSubject = TransformationSubject.initial(artifact.getId(), file);
-                    return transformationStep.transform(initialArtifactTransformationSubject, dependenciesResolver);
+                    return transformationStep.transform(initialArtifactTransformationSubject, dependenciesResolver, services);
                 }
 
                 @Override
@@ -149,29 +168,27 @@ public abstract class TransformationNode extends Node {
 
         @Override
         public void resolveDependencies(TaskDependencyResolver dependencyResolver, Action<Node> processHardSuccessor) {
+            super.resolveDependencies(dependencyResolver, processHardSuccessor);
             processDependencies(processHardSuccessor, dependencyResolver.resolveDependenciesFor(null, artifact));
-            processDependencies(processHardSuccessor, dependencyResolver.resolveDependenciesFor(null, dependenciesResolver.computeDependencyNodes(transformationStep)));
         }
 
     }
 
     private static class ChainedTransformationNode extends TransformationNode {
         private final TransformationNode previousTransformationNode;
-        private final ExecutionGraphDependenciesResolver dependenciesResolver;
 
         public ChainedTransformationNode(TransformationStep transformationStep, TransformationNode previousTransformationNode, ExecutionGraphDependenciesResolver dependenciesResolver) {
-            super(transformationStep);
+            super(transformationStep, dependenciesResolver);
             this.previousTransformationNode = previousTransformationNode;
-            this.dependenciesResolver = dependenciesResolver;
         }
 
         @Override
-        public void execute(BuildOperationExecutor buildOperationExecutor, ArtifactTransformListener transformListener) {
+        public void execute(BuildOperationExecutor buildOperationExecutor, ArtifactTransformListener transformListener, ProjectExecutionServiceRegistry services) {
             this.transformedSubject = buildOperationExecutor.call(new ArtifactTransformationStepBuildOperation() {
                 @Override
                 protected Try<TransformationSubject> transform() {
                     return previousTransformationNode.getTransformedSubject().flatMap(transformedSubject ->
-                        transformationStep.transform(transformedSubject, dependenciesResolver));
+                        transformationStep.transform(transformedSubject, dependenciesResolver, services));
                 }
 
                 @Override
@@ -185,9 +202,9 @@ public abstract class TransformationNode extends Node {
 
         @Override
         public void resolveDependencies(TaskDependencyResolver dependencyResolver, Action<Node> processHardSuccessor) {
+            super.resolveDependencies(dependencyResolver, processHardSuccessor);
             addDependencySuccessor(previousTransformationNode);
             processHardSuccessor.execute(previousTransformationNode);
-            processDependencies(processHardSuccessor, dependencyResolver.resolveDependenciesFor(null, dependenciesResolver.computeDependencyNodes(transformationStep)));
         }
 
     }
