@@ -16,6 +16,7 @@
 package org.gradle.internal.component.external.model;
 
 import com.google.common.base.Objects;
+import com.google.common.collect.ImmutableList;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.ModuleVersionSelector;
 import org.gradle.api.artifacts.VersionConstraint;
@@ -23,27 +24,34 @@ import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
 import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ImmutableVersionConstraint;
 import org.gradle.api.internal.artifacts.dependencies.DefaultImmutableVersionConstraint;
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 
+import java.util.Collection;
+import java.util.List;
+
 public class DefaultModuleComponentSelector implements ModuleComponentSelector {
     private final ModuleIdentifier moduleIdentifier;
     private final ImmutableVersionConstraint versionConstraint;
     private final ImmutableAttributes attributes;
+    private final ImmutableList<Capability> requestedCapabilities;
     private final int hashCode;
 
-    private DefaultModuleComponentSelector(ModuleIdentifier module, ImmutableVersionConstraint version, ImmutableAttributes attributes) {
+    private DefaultModuleComponentSelector(ModuleIdentifier module, ImmutableVersionConstraint version, ImmutableAttributes attributes, ImmutableList<Capability> requestedCapabilities) {
         assert module != null : "module cannot be null";
         assert version != null : "version cannot be null";
         assert attributes != null : "attributes cannot be null";
+        assert requestedCapabilities != null : "capabilities cannot be null";
         this.moduleIdentifier = module;
         this.versionConstraint = version;
         this.attributes = attributes;
+        this.requestedCapabilities = requestedCapabilities;
         // Do NOT change the order of members used in hash code here, it's been empirically
         // tested to reduce the number of collisions on a large dependency graph (performance test)
-        this.hashCode = Objects.hashCode(version, module, attributes);
+        this.hashCode = Objects.hashCode(version, module, attributes, requestedCapabilities);
     }
 
     public String getDisplayName() {
@@ -83,6 +91,11 @@ public class DefaultModuleComponentSelector implements ModuleComponentSelector {
         return moduleIdentifier;
     }
 
+    @Override
+    public List<Capability> getRequestedCapabilities() {
+        return requestedCapabilities;
+    }
+
     @Override
     public AttributeContainer getAttributes() {
         return attributes;
@@ -121,6 +134,9 @@ public class DefaultModuleComponentSelector implements ModuleComponentSelector {
         if (!attributes.equals(that.attributes)) {
             return false;
         }
+        if (!requestedCapabilities.equals(that.requestedCapabilities)) {
+            return false;
+        }
 
         return true;
     }
@@ -135,11 +151,12 @@ public class DefaultModuleComponentSelector implements ModuleComponentSelector {
         return getDisplayName();
     }
 
-    public static ModuleComponentSelector newSelector(ModuleIdentifier id, VersionConstraint version, AttributeContainer attributes) {
+    public static ModuleComponentSelector newSelector(ModuleIdentifier id, VersionConstraint version, AttributeContainer attributes, Collection<Capability> requestedCapabilities) {
         assert attributes != null : "attributes cannot be null";
         assert version != null : "version cannot be null";
+        assert requestedCapabilities != null : "capabilities cannot be null";
         assertModuleIdentifier(id);
-        return new DefaultModuleComponentSelector(id, DefaultImmutableVersionConstraint.of(version), ((AttributeContainerInternal)attributes).asImmutable());
+        return new DefaultModuleComponentSelector(id, DefaultImmutableVersionConstraint.of(version), ((AttributeContainerInternal)attributes).asImmutable(), ImmutableList.copyOf(requestedCapabilities));
     }
 
     private static void assertModuleIdentifier(ModuleIdentifier id) {
@@ -150,15 +167,15 @@ public class DefaultModuleComponentSelector implements ModuleComponentSelector {
     public static ModuleComponentSelector newSelector(ModuleIdentifier id, VersionConstraint version) {
         assert version != null : "version cannot be null";
         assertModuleIdentifier(id);
-        return new DefaultModuleComponentSelector(id, DefaultImmutableVersionConstraint.of(version), ImmutableAttributes.EMPTY);
+        return new DefaultModuleComponentSelector(id, DefaultImmutableVersionConstraint.of(version), ImmutableAttributes.EMPTY, ImmutableList.of());
     }
 
     public static ModuleComponentSelector newSelector(ModuleIdentifier id, String version) {
         assertModuleIdentifier(id);
-        return new DefaultModuleComponentSelector(id, DefaultImmutableVersionConstraint.of(version), ImmutableAttributes.EMPTY);
+        return new DefaultModuleComponentSelector(id, DefaultImmutableVersionConstraint.of(version), ImmutableAttributes.EMPTY, ImmutableList.of());
     }
 
     public static ModuleComponentSelector newSelector(ModuleVersionSelector selector) {
-        return new DefaultModuleComponentSelector(selector.getModule(), DefaultImmutableVersionConstraint.of(selector.getVersion()), ImmutableAttributes.EMPTY);
+        return new DefaultModuleComponentSelector(selector.getModule(), DefaultImmutableVersionConstraint.of(selector.getVersion()), ImmutableAttributes.EMPTY, ImmutableList.of());
     }
 }
