@@ -109,6 +109,12 @@ public abstract class DelegatingMavenModule<T extends MavenModule> implements Ma
         return t();
     }
 
+    @Override
+    public MavenModule withGradleMetadataRedirection() {
+        backingModule.withGradleMetadataRedirection();
+        return t();
+    }
+
     @Override
     public TestFile getArtifactFile() {
         return backingModule.getArtifactFile();
@@ -217,6 +223,12 @@ public abstract class DelegatingMavenModule<T extends MavenModule> implements Ma
         return t();
     }
 
+    @Override
+    public MavenModule adhocVariants() {
+        backingModule.adhocVariants();
+        return t();
+    }
+
     @Override
     public T parent(String group, String artifactId, String version) {
         backingModule.parent(group, artifactId, version);
