@@ -22,6 +22,7 @@ import org.gradle.api.XmlProvider;
 import org.gradle.api.internal.UserCodeAction;
 import org.gradle.api.internal.artifacts.Module;
 import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.publish.internal.versionmapping.VersionMappingStrategyInternal;
 import org.gradle.api.publish.ivy.IvyArtifact;
 import org.gradle.api.publish.ivy.IvyModuleDescriptorAuthor;
 import org.gradle.api.publish.ivy.IvyConfiguration;
@@ -149,6 +150,16 @@ public class DefaultIvyModuleDescriptorSpec implements IvyModuleDescriptorSpecIn
         return description;
     }
 
+    @Override
+    public VersionMappingStrategyInternal getVersionMappingStrategy() {
+        return ivyPublication.getVersionMappingStrategy();
+    }
+
+    @Override
+    public boolean writeGradleMetadataMarker() {
+        return ivyPublication.writeGradleMetadataMarker();
+    }
+
     private <T> void configureAndAdd(Class<? extends T> clazz, Action<? super T> action, List<T> items) {
         T item = instantiator.newInstance(clazz, objectFactory);
         action.execute(item);
