@@ -17,18 +17,19 @@ package org.gradle.testing.jacoco.plugins;
 
 import org.apache.commons.lang.StringUtils;
 import org.gradle.api.Action;
-import org.gradle.api.Incubating;
 import org.gradle.api.Plugin;
 import org.gradle.api.Project;
+import org.gradle.api.Task;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.DependencySet;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.plugins.JavaPlugin;
-import org.gradle.api.plugins.JavaPluginConvention;
 import org.gradle.api.plugins.ReportingBasePlugin;
 import org.gradle.api.reporting.ConfigurableReport;
 import org.gradle.api.reporting.Report;
 import org.gradle.api.reporting.ReportingExtension;
+import org.gradle.api.tasks.SourceSetContainer;
+import org.gradle.api.tasks.TaskProvider;
 import org.gradle.api.tasks.testing.Test;
 import org.gradle.internal.jacoco.JacocoAgentJar;
 import org.gradle.internal.reflect.Instantiator;
@@ -37,6 +38,7 @@ import org.gradle.testing.jacoco.tasks.JacocoBase;
 import org.gradle.testing.jacoco.tasks.JacocoCoverageVerification;
 import org.gradle.testing.jacoco.tasks.JacocoMerge;
 import org.gradle.testing.jacoco.tasks.JacocoReport;
+import org.gradle.util.DeprecationLogger;
 
 import javax.inject.Inject;
 import java.io.File;
@@ -45,14 +47,13 @@ import java.util.concurrent.Callable;
 /**
  * Plugin that provides support for generating Jacoco coverage data.
  */
-@Incubating
 public class JacocoPlugin implements Plugin<ProjectInternal> {
 
     /**
      * The jacoco version used if none is explicitly specified.
      * @since 3.4
      */
-    public static final String DEFAULT_JACOCO_VERSION = "0.8.1";
+    public static final String DEFAULT_JACOCO_VERSION = "0.8.3";
     public static final String AGENT_CONFIGURATION_NAME = "jacocoAgent";
     public static final String ANT_CONFIGURATION_NAME = "jacocoAnt";
     public static final String PLUGIN_EXTENSION_NAME = "jacoco";
@@ -82,7 +83,12 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
         configureAgentDependencies(agent, extension);
         configureTaskClasspathDefaults(extension);
         applyToDefaultTasks(extension);
-        configureDefaultOutputPathForJacocoMerge();
+        DeprecationLogger.whileDisabled(new Runnable() {
+            @Override
+            public void run() {
+                configureDefaultOutputPathForJacocoMerge();
+            }
+        });
         configureJacocoReportsDefaults(extension);
         addDefaultReportAndCoverageVerificationTasks(extension);
     }
@@ -126,7 +132,7 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
      */
     private void configureTaskClasspathDefaults(final JacocoPluginExtension extension) {
         final Configuration config = this.project.getConfigurations().getAt(ANT_CONFIGURATION_NAME);
-        project.getTasks().withType(JacocoBase.class, new Action<JacocoBase>() {
+        project.getTasks().withType(JacocoBase.class).configureEach(new Action<JacocoBase>() {
             @Override
             public void execute(JacocoBase task) {
                 task.setJacocoClasspath(config);
@@ -146,7 +152,7 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
      * @param extension the extension to apply Jacoco with
      */
     private void applyToDefaultTasks(final JacocoPluginExtension extension) {
-        project.getTasks().withType(Test.class, new Action<Test>() {
+        project.getTasks().withType(Test.class).configureEach(new Action<Test>() {
             @Override
             public void execute(Test task) {
                 extension.applyTo(task);
@@ -154,8 +160,16 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
         });
     }
 
+    /**
+     * Don't use this.
+     *
+     * @deprecated This method will be made private in a future version of Gradle.
+     */
+    @Deprecated
     public Object configureDefaultOutputPathForJacocoMerge() {
-        return project.getTasks().withType(JacocoMerge.class, new Action<JacocoMerge>() {
+        DeprecationLogger
+            .nagUserOfDiscontinuedMethod("JacocoPlugin.configureDefaultOutputPathForJacocoMerge()");
+        project.getTasks().withType(JacocoMerge.class).configureEach(new Action<JacocoMerge>() {
             @Override
             public void execute(final JacocoMerge task) {
                 task.setDestinationFile(project.provider(new Callable<File>() {
@@ -166,11 +180,11 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
                 }));
             }
         });
+        return project.getTasks().withType(JacocoMerge.class);
     }
 
-
     private void configureJacocoReportsDefaults(final JacocoPluginExtension extension) {
-        project.getTasks().withType(JacocoReport.class, new Action<JacocoReport>() {
+        project.getTasks().withType(JacocoReport.class).configureEach(new Action<JacocoReport>() {
             @Override
             public void execute(JacocoReport reportTask) {
                 configureJacocoReportDefaults(extension, reportTask);
@@ -178,16 +192,10 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
         });
     }
 
-    private void configureJacocoReportDefaults(final JacocoPluginExtension extension, final JacocoReport reportTask) {
-        reportTask.getReports().all(new Action<ConfigurableReport>() {
+    private Action<ConfigurableReport> configureReportOutputDirectory(final JacocoPluginExtension extension, final JacocoReport reportTask) {
+        return new Action<ConfigurableReport>() {
             @Override
             public void execute(final ConfigurableReport report) {
-                report.setEnabled(project.provider(new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        return report.getName().equals("html");
-                    }
-                }));
                 if (report.getOutputType().equals(Report.OutputType.DIRECTORY)) {
                     report.setDestination(project.provider(new Callable<File>() {
                         @Override
@@ -204,7 +212,22 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
                     }));
                 }
             }
+        };
+    }
+
+    private void configureJacocoReportDefaults(final JacocoPluginExtension extension, final JacocoReport reportTask) {
+        reportTask.getReports().all(new Action<ConfigurableReport>() {
+            @Override
+            public void execute(final ConfigurableReport report) {
+                report.setEnabled(project.provider(new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        return report.getName().equals("html");
+                    }
+                }));
+            }
         });
+        reportTask.getReports().all(configureReportOutputDirectory(extension, reportTask));
     }
 
     /**
@@ -216,52 +239,67 @@ public class JacocoPlugin implements Plugin<ProjectInternal> {
         project.getPlugins().withType(JavaPlugin.class, new Action<JavaPlugin>() {
             @Override
             public void execute(JavaPlugin javaPlugin) {
-                project.getTasks().withType(Test.class, new Action<Test>() {
-                    @Override
-                    public void execute(Test task) {
-                        if (task.getName().equals(JavaPlugin.TEST_TASK_NAME)) {
-                            addDefaultReportTask(extension, task);
-                            addDefaultCoverageVerificationTask(task);
-                        }
-                    }
-                });
+                final TaskProvider<Task> testTaskProvider = project.getTasks().named(JavaPlugin.TEST_TASK_NAME);
+                addDefaultReportTask(extension, testTaskProvider);
+                addDefaultCoverageVerificationTask(testTaskProvider);
             }
         });
     }
 
-    private void addDefaultReportTask(final JacocoPluginExtension extension, final Test task) {
-        final JacocoReport reportTask = project.getTasks().create("jacoco" + StringUtils.capitalize(task.getName()) + "Report", JacocoReport.class);
-        reportTask.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
-        reportTask.setDescription(String.format("Generates code coverage report for the %s task.", task.getName()));
-        reportTask.executionData(task);
-        reportTask.sourceSets(project.getConvention().getPlugin(JavaPluginConvention.class).getSourceSets().getByName("main"));
-        reportTask.getReports().all(new Action<ConfigurableReport>() {
-            @Override
-            public void execute(final ConfigurableReport report) {
-                if (report.getOutputType().equals(Report.OutputType.DIRECTORY)) {
-                    report.setDestination(project.provider(new Callable<File>() {
+    private void addDefaultReportTask(final JacocoPluginExtension extension, final TaskProvider<Task> testTaskProvider) {
+        project.getTasks().register(
+            "jacoco" + StringUtils.capitalize(testTaskProvider.getName()) + "Report",
+            JacocoReport.class,
+            new Action<JacocoReport>() {
+                @Override
+                public void execute(final JacocoReport reportTask) {
+                    reportTask.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
+                    reportTask.setDescription(String.format("Generates code coverage report for the %s task.", testTaskProvider.getName()));
+                    reportTask.executionData(testTaskProvider.get());
+                    reportTask.sourceSets(project.getExtensions().getByType(SourceSetContainer.class).getByName("main"));
+                    // TODO: Change the default location for these reports to follow the convention defined in #configureReportOutputDirectory
+                    reportTask.getReports().all(new Action<ConfigurableReport>() {
                         @Override
-                        public File call() throws Exception {
-                            return new File(extension.getReportsDir(), task.getName() + "/" + report.getName());
+                        public void execute(final ConfigurableReport report) {
+                            /*
+                             * For someone looking for the difference between this and the duplicate code above
+                             * this one uses the `testTaskProvider` and the `reportTask`. The other just
+                             * uses the `reportTask`.
+                             * https://github.com/gradle/gradle/issues/6343
+                             */
+                            if (report.getOutputType().equals(Report.OutputType.DIRECTORY)) {
+                                report.setDestination(project.provider(new Callable<File>() {
+                                    @Override
+                                    public File call() throws Exception {
+                                        return new File(extension.getReportsDir(), testTaskProvider.getName() + "/" + report.getName());
+                                    }
+                                }));
+                            } else {
+                                report.setDestination(project.provider(new Callable<File>() {
+                                    @Override
+                                    public File call() throws Exception {
+                                        return new File(extension.getReportsDir(), testTaskProvider.getName() + "/" + reportTask.getName() + "." + report.getName());
+                                    }
+                                }));
+                            }
                         }
-                    }));
-                } else {
-                    report.setDestination(project.provider(new Callable<File>() {
-                        @Override
-                        public File call() throws Exception {
-                            return new File(extension.getReportsDir(), task.getName() + "/" + reportTask.getName() + "." + report.getName());
-                        }
-                    }));
+                    });
                 }
-            }
-        });
+            });
     }
 
-    private void addDefaultCoverageVerificationTask(final Test task) {
-        final JacocoCoverageVerification coverageVerificationTask = project.getTasks().create("jacoco" + StringUtils.capitalize(task.getName()) + "CoverageVerification", JacocoCoverageVerification.class);
-        coverageVerificationTask.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
-        coverageVerificationTask.setDescription(String.format("Verifies code coverage metrics based on specified rules for the %s task.", task.getName()));
-        coverageVerificationTask.executionData(task);
-        coverageVerificationTask.sourceSets(project.getConvention().getPlugin(JavaPluginConvention.class).getSourceSets().getByName("main"));
+    private void addDefaultCoverageVerificationTask(final TaskProvider<Task> testTaskProvider) {
+        project.getTasks().register(
+            "jacoco" + StringUtils.capitalize(testTaskProvider.getName()) + "CoverageVerification",
+            JacocoCoverageVerification.class,
+            new Action<JacocoCoverageVerification>() {
+                @Override
+                public void execute(final JacocoCoverageVerification coverageVerificationTask) {
+                    coverageVerificationTask.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
+                    coverageVerificationTask.setDescription(String.format("Verifies code coverage metrics based on specified rules for the %s task.", testTaskProvider.getName()));
+                    coverageVerificationTask.executionData(testTaskProvider.get());
+                    coverageVerificationTask.sourceSets(project.getExtensions().getByType(SourceSetContainer.class).getByName("main"));
+                }
+            });
     }
 }
