@@ -17,17 +17,16 @@
 package org.gradle.api.internal.tasks.compile;
 
 import org.gradle.api.internal.ClassPathRegistry;
-import org.gradle.api.internal.file.IdentityFileResolver;
-import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.internal.tasks.compile.daemon.DaemonGroovyCompiler;
 import org.gradle.api.internal.tasks.compile.processing.AnnotationProcessorDetector;
 import org.gradle.api.tasks.WorkResult;
 import org.gradle.api.tasks.compile.GroovyCompileOptions;
-import org.gradle.internal.file.PathToFileResolver;
+import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.jvm.inspection.JvmVersionDetector;
 import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.language.base.internal.compile.CompilerFactory;
 import org.gradle.process.internal.DefaultExecActionFactory;
+import org.gradle.process.internal.JavaForkOptionsFactory;
 import org.gradle.process.internal.worker.child.WorkerDirectoryProvider;
 import org.gradle.workers.internal.IsolatedClassloaderWorkerFactory;
 import org.gradle.workers.internal.WorkerDaemonFactory;
@@ -36,20 +35,22 @@ import org.gradle.workers.internal.WorkerFactory;
 import java.io.Serializable;
 
 public class GroovyCompilerFactory implements CompilerFactory<GroovyJavaJointCompileSpec> {
-    private final ProjectInternal project;
     private final WorkerDaemonFactory workerDaemonFactory;
     private final IsolatedClassloaderWorkerFactory inProcessWorkerFactory;
-    private final PathToFileResolver fileResolver;
+    private final JavaForkOptionsFactory forkOptionsFactory;
     private final AnnotationProcessorDetector processorDetector;
     private final JvmVersionDetector jvmVersionDetector;
+    private final WorkerDirectoryProvider workerDirectoryProvider;
+    private final ClassPathRegistry classPathRegistry;
 
-    public GroovyCompilerFactory(ProjectInternal project, WorkerDaemonFactory workerDaemonFactory, IsolatedClassloaderWorkerFactory inProcessWorkerFactory, PathToFileResolver fileResolver, AnnotationProcessorDetector processorDetector, JvmVersionDetector jvmVersionDetector) {
-        this.project = project;
+    public GroovyCompilerFactory(WorkerDaemonFactory workerDaemonFactory, IsolatedClassloaderWorkerFactory inProcessWorkerFactory, JavaForkOptionsFactory forkOptionsFactory, AnnotationProcessorDetector processorDetector, JvmVersionDetector jvmVersionDetector, WorkerDirectoryProvider workerDirectoryProvider, ClassPathRegistry classPathRegistry) {
         this.workerDaemonFactory = workerDaemonFactory;
         this.inProcessWorkerFactory = inProcessWorkerFactory;
-        this.fileResolver = fileResolver;
+        this.forkOptionsFactory = forkOptionsFactory;
         this.processorDetector = processorDetector;
         this.jvmVersionDetector = jvmVersionDetector;
+        this.workerDirectoryProvider = workerDirectoryProvider;
+        this.classPathRegistry = classPathRegistry;
     }
 
     @Override
@@ -61,14 +62,14 @@ public class GroovyCompilerFactory implements CompilerFactory<GroovyJavaJointCom
         } else {
             workerFactory = inProcessWorkerFactory;
         }
-        Compiler<GroovyJavaJointCompileSpec> groovyCompiler = new DaemonGroovyCompiler(project.getServices().get(WorkerDirectoryProvider.class).getWorkingDirectory(), new DaemonSideCompiler(), project.getServices().get(ClassPathRegistry.class), workerFactory, fileResolver, jvmVersionDetector);
+        Compiler<GroovyJavaJointCompileSpec> groovyCompiler = new DaemonGroovyCompiler(workerDirectoryProvider.getWorkingDirectory(), new DaemonSideCompiler(), classPathRegistry, workerFactory, forkOptionsFactory, jvmVersionDetector);
         return new AnnotationProcessorDiscoveringCompiler<GroovyJavaJointCompileSpec>(new NormalizingGroovyCompiler(groovyCompiler), processorDetector);
     }
 
     private static class DaemonSideCompiler implements Compiler<GroovyJavaJointCompileSpec>, Serializable {
         @Override
         public WorkResult execute(GroovyJavaJointCompileSpec spec) {
-            DefaultExecActionFactory execHandleFactory = new DefaultExecActionFactory(new IdentityFileResolver());
+            DefaultExecActionFactory execHandleFactory = DefaultExecActionFactory.root();
             try {
                 Compiler<JavaCompileSpec> javaCompiler;
                 if (CommandLineJavaCompileSpec.class.isAssignableFrom(spec.getClass())) {
@@ -79,7 +80,7 @@ public class GroovyCompilerFactory implements CompilerFactory<GroovyJavaJointCom
                 Compiler<GroovyJavaJointCompileSpec> groovyCompiler = new ApiGroovyCompiler(javaCompiler);
                 return groovyCompiler.execute(spec);
             } finally {
-                execHandleFactory.stop();
+                CompositeStoppable.stoppable(execHandleFactory).stop();
             }
         }
     }
