@@ -15,7 +15,6 @@
  */
 package org.gradle.language.jvm.tasks;
 
-import org.gradle.api.Incubating;
 import org.gradle.api.tasks.Copy;
 import org.gradle.language.base.internal.tasks.SimpleStaleClassCleaner;
 import org.gradle.language.base.internal.tasks.StaleClassCleaner;
@@ -24,13 +23,12 @@ import org.gradle.language.base.internal.tasks.StaleClassCleaner;
  * Copies resources from their source to their target directory, potentially processing them.
  * Makes sure no stale resources remain in the target directory.
  */
-@Incubating
 public class ProcessResources extends Copy {
 
     @Override
     protected void copy() {
         StaleClassCleaner cleaner = new SimpleStaleClassCleaner(getOutputs());
-        cleaner.setDestinationDir(getDestinationDir());
+        cleaner.addDirToClean(getDestinationDir());
         cleaner.execute();
         super.copy();
     }
