@@ -23,11 +23,10 @@ import org.gradle.api.file.ConfigurableFileCollection;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.provider.Property;
-import org.gradle.api.provider.SetProperty;
 import org.gradle.language.BinaryCollection;
 import org.gradle.language.ComponentWithBinaries;
 import org.gradle.language.ComponentWithDependencies;
-import org.gradle.nativeplatform.TargetMachine;
+import org.gradle.language.ComponentWithTargetMachines;
 
 /**
  * Configuration for a C++ component, such as a library or executable, defining the source files and private header directories that make up the component. Private headers are those that are visible only to the source files of the component.
@@ -39,7 +38,7 @@ import org.gradle.nativeplatform.TargetMachine;
  * @since 4.2
  */
 @Incubating
-public interface CppComponent extends ComponentWithBinaries, ComponentWithDependencies {
+public interface CppComponent extends ComponentWithBinaries, ComponentWithDependencies, ComponentWithTargetMachines {
     /**
      * Specifies the base name for this component. This name is used to calculate various output file names. The default value is calculated from the project name.
      */
@@ -95,16 +94,4 @@ public interface CppComponent extends ComponentWithBinaries, ComponentWithDepend
      * @since 4.5
      */
     BinaryCollection<? extends CppBinary> getBinaries();
-
-    /**
-     * Specifies the target machines this component should be built for.  The "machines" extension property (see {@link org.gradle.nativeplatform.TargetMachineFactory}) can be used to construct common operating system and architecture combinations.
-     *
-     * <p>For example:</p>
-     * <pre>
-     * targetMachines = [machines.linux.x86_64, machines.windows.x86_64]
-     * </pre>
-     *
-     * @since 5.1
-     */
-    SetProperty<TargetMachine> getTargetMachines();
 }
