@@ -17,6 +17,7 @@
 package org.gradle.language.nativeplatform;
 
 import org.gradle.api.Incubating;
+import org.gradle.api.Task;
 import org.gradle.api.file.RegularFile;
 import org.gradle.api.provider.Provider;
 
@@ -27,6 +28,13 @@ import org.gradle.api.provider.Provider;
  */
 @Incubating
 public interface ComponentWithLinkFile extends ComponentWithNativeRuntime {
+    /**
+     * Returns the task that should be run to produce the link file of this component. This isn't necessarily the link task for the component.
+     *
+     * @since 5.1
+     */
+    Provider<? extends Task> getLinkFileProducer();
+
     /**
      * Returns the link file of this component.
      */
