@@ -15,7 +15,7 @@
  */
 package org.gradle.launcher.bootstrap;
 
-import org.gradle.BuildExceptionReporter;
+import org.gradle.internal.buildevents.BuildExceptionReporter;
 import org.gradle.api.Action;
 import org.gradle.configuration.GradleLauncherMetaData;
 import org.gradle.internal.logging.DefaultLoggingConfiguration;
@@ -24,7 +24,7 @@ import org.gradle.internal.logging.text.StreamingStyledTextOutputFactory;
 /**
  * An entry point is the point at which execution will never return from.
  * <p>
- * It's purpose is to consistently apply our completion logic of forcing the JVM
+ * Its purpose is to consistently apply our completion logic of forcing the JVM
  * to exit at a certain point instead of waiting for all threads to die, and to provide
  * some consistent unhandled exception catching.
  * <p>
@@ -79,4 +79,4 @@ public abstract class EntryPoint {
         }
     }
 
-}
\ No newline at end of file
+}
