@@ -17,6 +17,7 @@ package org.gradle.launcher.daemon.client;
 
 import com.google.common.base.Preconditions;
 import org.gradle.api.internal.specs.ExplainingSpec;
+import org.gradle.api.internal.specs.ExplainingSpecs;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
 import org.gradle.api.specs.Spec;
@@ -27,6 +28,7 @@ import org.gradle.internal.logging.progress.ProgressLoggerFactory;
 import org.gradle.internal.remote.internal.ConnectException;
 import org.gradle.internal.remote.internal.OutgoingConnector;
 import org.gradle.internal.remote.internal.RemoteConnection;
+import org.gradle.internal.serialize.Serializer;
 import org.gradle.internal.serialize.Serializers;
 import org.gradle.internal.time.CountdownTimer;
 import org.gradle.internal.time.Time;
@@ -34,7 +36,6 @@ import org.gradle.launcher.daemon.context.DaemonConnectDetails;
 import org.gradle.launcher.daemon.context.DaemonContext;
 import org.gradle.launcher.daemon.diagnostics.DaemonStartupInfo;
 import org.gradle.launcher.daemon.logging.DaemonMessages;
-import org.gradle.launcher.daemon.protocol.DaemonMessageSerializer;
 import org.gradle.launcher.daemon.protocol.Message;
 import org.gradle.launcher.daemon.registry.DaemonInfo;
 import org.gradle.launcher.daemon.registry.DaemonRegistry;
@@ -64,9 +65,11 @@ public class DefaultDaemonConnector implements DaemonConnector {
     private final DaemonStarter daemonStarter;
     private final DaemonStartListener startListener;
     private final ProgressLoggerFactory progressLoggerFactory;
+    private final Serializer<Message> serializer;
     private long connectTimeout = DefaultDaemonConnector.DEFAULT_CONNECT_TIMEOUT;
 
-    public DefaultDaemonConnector(DaemonRegistry daemonRegistry, OutgoingConnector connector, DaemonStarter daemonStarter, DaemonStartListener startListener, ProgressLoggerFactory progressLoggerFactory) {
+    public DefaultDaemonConnector(DaemonRegistry daemonRegistry, OutgoingConnector connector, DaemonStarter daemonStarter, DaemonStartListener startListener, ProgressLoggerFactory progressLoggerFactory, Serializer<Message> serializer) {
+        this.serializer = serializer;
         Preconditions.checkNotNull(daemonRegistry);
         Preconditions.checkNotNull(connector);
         Preconditions.checkNotNull(daemonStarter);
@@ -200,9 +203,13 @@ public class DefaultDaemonConnector implements DaemonConnector {
     }
 
     public DaemonClientConnection startDaemon(ExplainingSpec<DaemonContext> constraint) {
+        return doStartDaemon(constraint, false);
+    }
+
+    private DaemonClientConnection doStartDaemon(ExplainingSpec<DaemonContext> constraint, boolean singleRun) {
         ProgressLogger progressLogger = progressLoggerFactory.newOperation(DefaultDaemonConnector.class)
             .start("Starting Gradle Daemon", "Starting Daemon");
-        final DaemonStartupInfo startupInfo = daemonStarter.startDaemon();
+        final DaemonStartupInfo startupInfo = daemonStarter.startDaemon(singleRun);
         LOGGER.debug("Started Gradle daemon {}", startupInfo);
         CountdownTimer timer = Time.startCountdownTimer(connectTimeout);
         try {
@@ -225,6 +232,11 @@ public class DefaultDaemonConnector implements DaemonConnector {
         throw new DaemonConnectionException("Timeout waiting to connect to the Gradle daemon.\n" + startupInfo.describe());
     }
 
+    @Override
+    public DaemonClientConnection startSingleUseDaemon() {
+        return doStartDaemon(ExplainingSpecs.<DaemonContext>satisfyAll(), true);
+    }
+
     private DaemonClientConnection connectToDaemonWithId(DaemonStartupInfo daemon, ExplainingSpec<DaemonContext> constraint) throws ConnectException {
         // Look for 'our' daemon among the busy daemons - a daemon will start in busy state so that nobody else will grab it.
         for (DaemonInfo daemonInfo : daemonRegistry.getNotIdle()) {
@@ -249,7 +261,7 @@ public class DefaultDaemonConnector implements DaemonConnector {
             .start("Connecting to Gradle Daemon", "Connecting to Daemon");
         RemoteConnection<Message> connection;
         try {
-            connection = connector.connect(daemon.getAddress()).create(Serializers.stateful(DaemonMessageSerializer.create()));
+            connection = connector.connect(daemon.getAddress()).create(Serializers.stateful(serializer));
         } catch (ConnectException e) {
             staleAddressDetector.maybeStaleAddress(e);
             throw e;
