@@ -19,12 +19,28 @@ package org.gradle.api.internal.provider;
 import org.gradle.api.Transformer;
 import org.gradle.api.internal.tasks.TaskDependencyResolveContext;
 import org.gradle.api.provider.Provider;
+import org.gradle.internal.state.Managed;
 import org.gradle.util.GUtil;
 
 import javax.annotation.Nullable;
 import java.util.Collection;
 
-public abstract class AbstractMinimalProvider<T> implements ProviderInternal<T> {
+public abstract class AbstractMinimalProvider<T> implements ProviderInternal<T>, Managed {
+    private static final Factory FACTORY = new Factory() {
+        @Nullable
+        @Override
+        public <T> T fromState(Class<T> type, Object state) {
+            if (!type.isAssignableFrom(Provider.class)) {
+                return null;
+            }
+            if (state == null) {
+                return type.cast(Providers.notDefined());
+            } else {
+                return type.cast(Providers.of(state));
+            }
+        }
+    };
+
     @Override
     public <S> ProviderInternal<S> map(final Transformer<? extends S, ? super T> transformer) {
         return new TransformBackedProvider<S, T>(transformer, this);
@@ -85,6 +101,26 @@ public abstract class AbstractMinimalProvider<T> implements ProviderInternal<T>
         return String.format("provider(%s)", GUtil.elvis(getType(), "?"));
     }
 
+    @Override
+    public boolean immutable() {
+        return false;
+    }
+
+    @Override
+    public Class<?> publicType() {
+        return Provider.class;
+    }
+
+    @Override
+    public Factory managedFactory() {
+        return FACTORY;
+    }
+
+    @Override
+    public Object unpackState() {
+        return getOrNull();
+    }
+
     private static class FlatMapProvider<S, T> extends AbstractMinimalProvider<S> {
         private final Provider<? extends T> provider;
         private final Transformer<? extends Provider<? extends S>, ? super T> transformer;
