@@ -19,19 +19,15 @@ package org.gradle.internal.reflect;
 import javax.annotation.Nullable;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Method;
-import java.util.List;
 
 public interface PropertyMetadata {
-    String getFieldName();
+    String getPropertyName();
 
     boolean isAnnotationPresent(Class<? extends Annotation> annotationType);
 
     @Nullable
     <A extends Annotation> A getAnnotation(Class<A> annotationType);
 
-    List<String> getValidationMessages();
-
-    @Nullable
     Class<? extends Annotation> getPropertyType();
 
     Class<?> getDeclaredType();
