@@ -26,7 +26,6 @@ import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.PublishArtifact;
 import org.gradle.api.artifacts.repositories.IvyArtifactRepository;
-import org.gradle.api.internal.ConventionMapping;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.Module;
 import org.gradle.api.internal.artifacts.configurations.ConfigurationInternal;
@@ -44,8 +43,8 @@ import org.gradle.internal.Describables;
 import org.gradle.jvm.tasks.Jar;
 import org.gradle.language.base.plugins.LifecycleBasePlugin;
 
+import javax.annotation.Nullable;
 import javax.inject.Inject;
-import java.io.File;
 import java.util.concurrent.Callable;
 
 /**
@@ -87,35 +86,35 @@ public class BasePlugin implements Plugin<Project> {
     private void configureArchiveDefaults(final Project project, final BasePluginConvention pluginConvention) {
         project.getTasks().withType(AbstractArchiveTask.class).configureEach(new Action<AbstractArchiveTask>() {
             public void execute(AbstractArchiveTask task) {
-                ConventionMapping taskConventionMapping = task.getConventionMapping();
 
-                Callable<File> destinationDir;
+                Callable<String> destinationDir;
                 if (task instanceof Jar) {
-                    destinationDir = new Callable<File>() {
-                        public File call() throws Exception {
-                            return pluginConvention.getLibsDir();
+                    destinationDir = new Callable<String>() {
+                        public String call() {
+                            return pluginConvention.getLibsDirName();
                         }
                     };
                 } else {
-                    destinationDir = new Callable<File>() {
-                        public File call() throws Exception {
-                            return pluginConvention.getDistsDir();
+                    destinationDir = new Callable<String>() {
+                        public String call() {
+                            return pluginConvention.getDistsDirName();
                         }
                     };
                 }
-                taskConventionMapping.map("destinationDir", destinationDir);
+                task.getDestinationDirectory().convention(project.getLayout().getBuildDirectory().dir(project.provider(destinationDir)));
 
-                taskConventionMapping.map("version", new Callable<String>() {
-                    public String call() throws Exception {
+                task.getArchiveVersion().convention(project.provider(new Callable<String>() {
+                    @Nullable
+                    public String call() {
                         return project.getVersion() == Project.DEFAULT_VERSION ? null : project.getVersion().toString();
                     }
-                });
+                }));
 
-                taskConventionMapping.map("baseName", new Callable<String>() {
-                    public String call() throws Exception {
+                task.getArchiveBaseName().convention(project.provider(new Callable<String>() {
+                    public String call() {
                         return pluginConvention.getArchivesBaseName();
                     }
-                });
+                }));
             }
         });
     }
@@ -129,8 +128,8 @@ public class BasePlugin implements Plugin<Project> {
     }
 
     private void configureUploadArchivesTask() {
-        configurationActionContainer.add(new Action<Project>() {
-            public void execute(Project project) {
+        configurationActionContainer.add(new Action<ProjectInternal>() {
+            public void execute(ProjectInternal project) {
                 Upload uploadArchives = project.getTasks().withType(Upload.class).findByName(UPLOAD_ARCHIVES_TASK_NAME);
                 if (uploadArchives == null) {
                     return;
@@ -144,7 +143,7 @@ public class BasePlugin implements Plugin<Project> {
                 ConfigurationInternal configuration = (ConfigurationInternal) uploadArchives.getConfiguration();
                 Module module = configuration.getModule();
                 ModuleVersionIdentifier publicationId = moduleIdentifierFactory.moduleWithVersion(module.getGroup(), module.getName(), module.getVersion());
-                publicationRegistry.registerPublication(module.getProjectPath(), new DefaultProjectPublication(Describables.of("Ivy publication"), publicationId, true));
+                publicationRegistry.registerPublication(project, new DefaultProjectPublication(Describables.of("Ivy publication"), publicationId, true));
             }
         });
     }
