@@ -16,7 +16,6 @@
 
 package org.gradle.api.reporting;
 
-import org.gradle.api.Incubating;
 import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.OutputDirectory;
 
@@ -25,7 +24,6 @@ import java.io.File;
 /**
  * A directory based report to be created.
  */
-@Incubating
 public interface DirectoryReport extends ConfigurableReport {
 
     /**
